import NumericEditor from './NumericEditor';
import {fireEvent, render} from "@testing-library/react";
import {removeReact18ConsoleErrors} from "../../utils/test.helper";

removeReact18ConsoleErrors()

describe('NumericEditor', () => {
    let props = {
        eventKey: '',
        value: '',
    };

    test('set initial state correctly based on props', () => {
        const startValue = '123';
        props.value = startValue
        const {getByTestId} = render(<NumericEditor {...props} />);

        const inputElement = getByTestId('numeric-editor-input');
        expect(inputElement.value).toBe(startValue);
    });

    test('update state value on input change', () => {
        const {getByTestId} = render(<NumericEditor {...props} />);
        const inputElement = getByTestId('numeric-editor-input');

        const newValue = '4.56';
        fireEvent.change(inputElement, {target: {value: newValue}});
        expect(inputElement.value).toBe(newValue);
    });

    //negative, non-numeric, keyboard actions and double dot
    test.each(['-', 'A', 'Enter', '.'])(`Prevent invalid key events entries: %s`, (eventKey) => {
        const {getByTestId} = render(<NumericEditor {...props} />);

        const inputElement = getByTestId('numeric-editor-input');

        const preventDefaultMock = jest.fn()
        let event = {
            preventDefault: preventDefaultMock(),
            target: {
                value: '1.5',
            },
        }

        event.key = eventKey
        fireEvent.keyDown(inputElement, {event});
        expect(preventDefaultMock).toHaveBeenCalled();
    })
});