
import { Link } from "react-router-dom";
import { Button } from 'primereact/button';
import { Dropdown } from 'primereact/dropdown';
import { Calendar } from 'primereact/calendar';
const PageActionMenu = ({ actions, className }) => {
    const onClickLink = (action) => {
        if (action.link) {
            action.link();
        }
    };

    const onButtonClick = (e, action) => {
        e.preventDefault()
        if (action.actOn && action.actOn === 'click') {
            action.props.callback(e);
        }
    };

    // we let the keydown be equal to a click event
    const handleOnKeyDown = (e, action) => {
        if (e.key === 'Enter') {
            if (action.actOn && action.actOn === 'click') {
                action.props.callback(e);
            }
        }
    }

    // Both Focus and mouseover will trigger mouseOver, for legacy reasons we leave this in FE1. 
    const onButtonMouseOver = (e, action) => {
        if (action.actOn && action.actOn === 'mouseOver') {
            action.props.callback(e);
        }
    }
    const setSelected = (e, action) => {
        if (action.actOn && action.actOn === 'select') {
            action.props.callback(e.value);
        }
    }


    return <div className={"page-action-menu " + className}>
        {(actions || []).map((action,index) => {
            switch (action.type) {

                   case 'buttonv2':
                    return (
                        <Button key={action.title + '_page_header'} title={action.title || ''}
                            style={action.style}
                            icon={`pi ${action.icon}`} disabled={action.disabled}
                            onMouseOver={(e) => action.disabled ? '' : onButtonMouseOver(e, action)}
                            onFocus={(e) => action.disabled ? '' : onButtonMouseOver(e, action)}
                            onClick={(e) => action.disabled ? '' : onButtonClick(e, action)}
                            onKeyDown={(e) => action.disabled ? '' : handleOnKeyDown(e, action)}
                        >
                        </Button>
                    );
                    
                case 'button':
                    return (
                        <button className="p-link" key={action.title + '_page_header'} title={action.title || ''}
                            style={action.style}>
                            <i className={`${action.iconType ? action.iconType : 'fa'} ${action.disabled ? 'fa-disabled' : ''} ${action.icon} ${action.classes}`}
                                onMouseOver={(e) => action.disabled ? '' : onButtonMouseOver(e, action)}
                                onFocus={(e) => action.disabled ? '' : onButtonMouseOver(e, action)}
                                onClick={(e) => action.disabled ? '' : onButtonClick(e, action)}
                                onKeyDown={(e) => action.disabled ? '' : handleOnKeyDown(e, action)} />
                        </button>
                    );
                case 'divider':
                    return (<span className="action-divider" key={index}></span>);
                case 'element':
                    return (
                        <div key={index} className={action.classes} dangerouslySetInnerHTML={{ __html: action.element }} />
                    )
                case 'dropdown':

                    if (action.optionsvalue) {
                            return (<div className={action.classes} key={action.title}> <span className="dropdown-title">{action.title} </span><Dropdown value={action.selected} onChange={(e) => setSelected(e, action)} options={action.options}  optionValue={action.optionvalue}  optionLabel={action.optionlabel} className="w-full md:w-14rem" /></div>)
                    }
                    else {
                        return (<div className={action.classes} key={action.title}> <span className="dropdown-title">{action.title} </span><Dropdown value={action.selected} onChange={(e) => setSelected(e, action)} options={action.options}   optionLabel={action.optionlabel} className="w-full md:w-14rem" placeholder={action.optionSelectionLabel}/></div>)
                    }
                case 'ext_link':
                    return (
                        <a href={action.props.pathname} title={action.title || ''} key={action.title}
                            target={action.target ? action.target : "_blank"} rel="noreferrer">{action.label}</a>
                    );
                case 'tag':
                    return (
                        <span key={action.title + '_page_header'} className={action.className}
                            title={action.title || ''}>{action.content}</span>
                    );
            case 'calendar':
                return (
                    <div key={`index`}><span>{action.title}</span> <Calendar value={action.selected} onChange={(e) => setSelected(e, action)} dateFormat="yy-mm-dd" showIcon className={action.className} /> </div>
                )

                default:
                    return (
                        <Link key={action.title + '_page_header'} className={action.classname}
                            to={action.disabled ? {} : { ...action.props }}
                            title={action.title || ''} onClick={() => action.disabled ? '' : onClickLink(action)}
                            style={action.style}>
                            <i className={`${action.iconType ? action.iconType : 'fa'} ${action.disabled ? 'fa-disabled' : ''} ${action.icon}`}></i>
                        </Link>
                    );
            }
        })}
    </div>
}

export default PageActionMenu;