import {useEffect, useState} from 'react';
import {routes} from '../../routes';
import {matchPath} from 'react-router-dom';
import PageActionMenu from "./PageActionMenu";
import PropTypes from 'prop-types';

const PageHeader = ({title, subTitle, actions, className, ...props}) => {
    
    const [page, setPage] = useState({});


    useEffect(() => {
        const currentRoute = routes.find(route => matchPath(props.location?.pathname, {
            path: route.path,
            exact: true,
            strict: true
        }));
        //for intial route ,there won't be any route object so it failed
        if (!currentRoute) {
            return;
        }
        setPage(currentRoute);
    }, [props.location?.pathname]);


    
    return (
        <div className={"page-header " + className || ""}>
            <div className="title">
                <h2 className="page-title">{title || page.title}</h2>
                {(page.subTitle || subTitle) && <h6 className="page-subtitle">{subTitle || page.subTitle}</h6>}
            </div>
            <PageActionMenu actions={actions} className={"page-actions" + className || ""} />
        </div>
    );
}

PageHeader.propTypes= {
    title:PropTypes.string,
    subTitle:PropTypes.string,
    actions:PropTypes.array,
    className:PropTypes.string,
    location:PropTypes.oneOfType([
        PropTypes.object,
        PropTypes.string,
      ])
}
export default PageHeader;