import CycleReportIntro from './report.intro';
import CycleTelescopeTime from './report.telescope.time';
import CycleReportAvgEfficiency from './report.avg.efficiency';
import CycleCompletionLevel from './report.cycle.completion';
import CycleCategoryObservations from './report.category.observations';
import CycleWeeklyEfficiency from './report.weekly.efficiency';
import CycleSiteWiseData from './report.site.data';
import CycleCategoryWiseData from './report.category.data';
import CycleProjectSummary from './report.project.summary';
import CyclesILTLocalUsage from './report.ilt_local.usage';
import CycleFailureRate from './report.failure.rate';
import CycleOverview from './report.overview';
import PropTypes from 'prop-types';
import ReportBlock from './report.block';


export default function CycleReportContent(props) {

    let { reportPeriod, reportCycles, ReportData } = props;
    /**
     * 
     * @returns margins for print report
     */
    const getPageMargins = () => {
        return `@page { margin: 138px 72px 138px 72px !important; }`;
    }



    return (<div >
        <style>{getPageMargins()}</style>
        <ReportBlock id="Intro">
            <CycleReportIntro reportPeriod={reportPeriod} cycles={reportCycles} />
        </ReportBlock>

        <ReportBlock id="Overview" title="Cycle Overview">
            <CycleOverview id="Overview" data={ReportData?.["Overview"]} />
        </ReportBlock>
        <ReportBlock id="TelescopeTime" title="Telescope Time Distribution">
            <CycleTelescopeTime id="TelescopeTime" data={ReportData?.["TelescopeTime"]} />
        </ReportBlock>
        <ReportBlock id="AverageEfficiency" title="Average Efficiency">
            <CycleReportAvgEfficiency data={ReportData?.["AverageEfficiency"]} />
        </ReportBlock>

        <ReportBlock id="CompletionLevel" title="Cycle Completion Level">
            <CycleCompletionLevel id="CompletionLevel" data={ReportData?.["CompletionLevel"]} />
        </ReportBlock>

        <ReportBlock id="CategoryObservation" title="Observation Hours per Category">
            <CycleCategoryObservations id="CategoryObservation" data={ReportData?.["CategoryObservation"]} />
        </ReportBlock>

        <ReportBlock id="WeeklyEfficiency" title="Weekly Efficiency">
            <CycleWeeklyEfficiency id="WeeklyEfficiency"  data={ReportData?.["WeeklyEfficiency"]} />
        </ReportBlock>
        <ReportBlock id="DataIngestedPerSite" title="Ingested Data per Site">
            <CycleSiteWiseData id="DataIngestedPerSite" data={ReportData?.["DataIngestedPerSite"]} />
        </ReportBlock>

        <ReportBlock id="DataIngestedPerCategory" title="Ingested Data per Category">
            <CycleCategoryWiseData id="DataIngestedPerCategory" data={ReportData?.["DataIngestedPerCategory"]} />
        </ReportBlock>

        <ReportBlock id="ProjectSummary" title="Projects Summary Overview">
            <CycleProjectSummary id="ProjectSummary" data={ReportData?.["ProjectSummary"]} />
        </ReportBlock>

        <ReportBlock id="ILT_LocalUsage" title="ILT vs Local Mode Usage">
            <CyclesILTLocalUsage id="ILT_LocalUsage" data={ReportData?.["ILT_LocalUsage"]} />
        </ReportBlock>

        <ReportBlock id="Failures" title="Failure Rate">
            <CycleFailureRate id="Failures" data={ReportData?.["Failures"]} />

        </ReportBlock>
    </div>
    )
}

CycleReportContent.propTypes = {
    reportPeriod: PropTypes.array,
    reportCycles: PropTypes.array,
    ReportData: PropTypes.object,

};