
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import PropTypes from 'prop-types';
import _ from 'lodash';

/**
 * Component to show data ingested per category in cycle(s).
 */
export default function CycleCompletionLevel(props) {
    let { data } = props;

    /**
     * Function that returns the data required for the chart with lablels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    const getBarData = () => {
        return {
            labels: _.map(data, 'cycle'),
            datasets: [
                {
                    type: 'line',
                    label: 'Target',
                    borderColor: '#ff0000',
                    backgroundColor: '#ff0000',
                    borderWidth: 2,
                    fill: false,
                    data: _.map(data, 'target')
                },
                {
                    type: 'line',
                    label: 'Prognosis',
                    borderColor: '#ffc107',
                    backgroundColor: '#ffc107',
                    borderWidth: 2,
                    fill: false,
                    data: _.map(data, 'prognosis')
                },
                {
                    type: 'bar',
                    label: 'Performed',
                    borderColor: '#44a3ce',
                    backgroundColor: '#44a3ce',
                    barThickness: 50,
                    // barPercentage: 0.3,
                    data: _.map(data, 'performed'),
                    borderWidth: 2
                }
            ],
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
    const getBarOptions = () => {
        return {
            // indexAxis: 'y',
            // Elements options apply to all of the options unless overridden in a dataset
            // In this case, we are setting the border of each horizontal bar to be 1px wide
            elements: {
                bar: {
                    borderWidth: 1,
                },
            },
            scales: {
                y: {
                    max: 100
                }
            },
            responsive: true,
            plugins: {
                legend: {
                    position: 'bottom',
                },
                title: {
                    display: true,
                    text: 'Completion Level LOFAR Observations',
                },
            }
        };
    }




    return (
        <>
            <Bar data={getBarData()} options={getBarOptions()} />
            <DataTable value={data} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                <Column field="cycle" header="Cycle code"></Column>
                <Column field="performed" header="Performed (%)"></Column>
                <Column field="prognosis" header="Prognosis (%)"></Column>
                <Column field="target" header="Target (%)"></Column>
            </DataTable>
        </>
    );
}

CycleCompletionLevel.propTypes = {
    data: PropTypes.array,
};