import PropTypes from 'prop-types';
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import _ from 'lodash';


/**
 * Failures rate report component for cycle report.
 */
export default function CycleFailureRate(props) {
    let { data } = props;

    /**
     * Function that returns the data required for the chart with labels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    const getBarData = () => {
        const barData = data.chartData;
        return {
            labels: _.map(barData, 'month'),
            datasets: [
                {
                    type: 'bar',
                    label: 'Hours Lost',
                    borderColor: '44a3ce',
                    backgroundColor: '#44a3ce',
                    borderWidth: 2,
                    barThickness: 50,
                    data: _.map(barData, 'duration'),
                }
            ],
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
    const getBarOptions = () => {
        return {
            elements: {
                bar: {
                    borderWidth: 1,
                },
            },
            responsive: true,
            plugins: {
                legend: {
                    position: 'bottom',
                },
                title: {
                    display: true,
                    text: 'LOFAR Observing Failures',
                },
            }
        };
    }

    return (
        <>
            {data &&
                <>
                    <Bar data={getBarData()} options={getBarOptions()} />
                    <DataTable value={data.monthTableData} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                        <Column field="cycle" header="Cycle Code" />
                        <Column field="month" header="Period (YYYY-MM)" />
                        <Column field="duration" header="Hours Lost per Month" />
                    </DataTable>
                    <DataTable value={data.cycleTableData} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                        <Column field="cycle" header="Cycle Code" />
                        <Column field="period" header="Period (YYYY-MM)" />
                        <Column field="duration" header="Hours Lost per Cycle" />
                    </DataTable>
                </>
            }
        </>
    );


}


CycleFailureRate.propTypes = {
    data: PropTypes.object,
};