import React, { Component } from 'react';
import { Redirect } from 'react-router-dom';
import moment from 'moment';
import _ from 'lodash';
import $RefParser from "@apidevtools/json-schema-ref-parser";

import { TriStateCheckbox } from 'primereact/tristatecheckbox';
import { InputText } from 'primereact/inputtext';
import { InputTextarea } from 'primereact/inputtextarea';
import { Dropdown } from 'primereact/dropdown';
import { Toast } from 'primereact/toast';
import { CustomDialog } from '../../layout/components/CustomDialog';
import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import Jeditor from '../../components/JSONEditor/JEditor';
import UnitConversion from '../../utils/unit.converter';
import Stations from './Stations';
import { publish } from '../../App';

import ProjectService from '../../services/project.service';
import ScheduleService from '../../services/schedule.service';
import TaskService from '../../services/task.service';
import UIConstants from '../../utils/ui.constants';
import ParserUtility from '../../utils/parser.utility';
import SchedulingConstraint from './Scheduling.Constraints';
import UtilService from '../../services/util.service';
import ReactTooltip from "react-tooltip";
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
import { appGrowl } from '../../layout/components/AppGrowl';
import FormActionbar from '../../components/FormActionbar'; 

/**
 * Compoenent to edit scheduling unit draft
 */
export class EditSchedulingUnit extends Component {
    constructor(props) {
        super(props);
        this.state = {
            showDialog: false,
            errorDialog: false,
            isDirty: false,
            isLoading: true,                        // Flag for loading spinner                     
            dialog: { header: '', detail: ''},      // Dialog properties
            redirect: null,                         // URL to redirect
            errors: [],                             // Form Validation errors
            schedulingSets: [],                     // Scheduling set of the selected project
            schedulingUnit: {},
            projectDisabled: (props.match?(props.match.params.project? true:false):false),      
            observStrategy: {},                     // Selected strategy to create SU
            paramsSchema: null,                     // JSON Schema to be generated from strategy template to pass to JSOn editor
            constraintSchema:null,                     
            tasksToUpdate: {},
            validEditor: false,                     // For JSON editor validation
            validFields: {},                        // For Form Validation 
            observStrategyVisible: false,
            missingStationFieldsErrors: [],         // Validation for max no.of missing station
            stationGroups: {},
            customSelectedStations: [],             // custom stations
            selectedStationsOfTask : {},
            missingStationsofTask: {},
            missingStnErorsofTask: {},
            customStationsOfTask: {},
            stationsCollapsed: false,
            permissionById: {}                      // Custom Stations    
        }
        this.taskFilters = [];                      // To get Short_Description details
        this.priorityQueueTypes = [];
        this.projects = [];                         // All projects to load project dropdown
        this.schedulingSets = [];                   // All scheduling sets to be filtered for project
        this.observStrategies = [];                 // All Observing strategy templates
        this.taskTemplates = [];                    // All task templates to be filtered based on tasks in selected strategy template             
        this.schedulingSets = [];                   
        //this.observStrategies = [];
        this.taskTemplates = [];                    
        this.constraintTemplates = [];
        this.tooltipOptions = UIConstants.tooltipOptions;
        this.nameInput = React.createRef();         // Ref to Name field for auto focus
        this.formRules = {                          // Form validation rules                  
            name: {required: true, message: "Name can not be empty"},
            description: {required: true, message: "Description can not be empty"},
        };
 
        this.setEditorOutput = this.setEditorOutput.bind(this);
        this.changeStrategy = this.changeStrategy.bind(this);
        this.constraintStrategy = this.constraintStrategy.bind(this);
        this.setSchedUnitParams = this.setSchedUnitParams.bind(this);
        this.validateForm = this.validateForm.bind(this);
        this.validateStations = this.validateStations.bind(this);
        this.validateEditor = this.validateEditor.bind(this);
        this.setEditorFunction = this.setEditorFunction.bind(this);
        this.saveSchedulingUnit = this.saveSchedulingUnit.bind(this);
        this.cancelCreate = this.cancelCreate.bind(this);
        this.closeErrorPopup= this.closeErrorPopup.bind(this);
        this.setEditorOutputConstraint = this.setEditorOutputConstraint.bind(this);
        this.checkIsDirty = this.checkIsDirty.bind(this);
        this.close = this.close.bind(this);
    }

    closeErrorPopup(){
        this.setState({errorDialog :false});
    }
    
    /**
     * Function called when observation strategy template is changed. 
     * It generates the JSON schema for JSON editor and defult vales for the parameters to be captured
     * @param {number} strategyId 
     */
     async changeStrategy (strategyId) {
        const observStrategy = _.find(this.observStrategies, {'id': strategyId});
        ParserUtility.addStationParameters(observStrategy);
        let stationGroups = {};
        if (observStrategy) {
            let tasksToUpdate = {};
            const tasks = observStrategy.template.tasks;
            const parameters = observStrategy.template.parameters;
            let paramsOutput = {};
            let schema = { type: 'object', additionalProperties: false, 
                            properties: {}, definitions:{}
                            };
            let bandPassFilter = null;
            const $strategyRefs = await $RefParser.resolve(observStrategy.template);
            // TODo: This schema reference resolving code has to be moved to common file and needs to rework
            for (const param of parameters) {
                // TODO: make parameter handling more generic, instead of task specific.
                if (!param.refs[0].startsWith("#/tasks/")) { continue; }
                let taskPaths = param.refs[0].split("/");
                const taskName = taskPaths[2];
                /**
                 * For Short_Description, the task path length will be 4, so added below condition to get short_description details
                 *  #/tasks/Combined Observation/short_description
                */
                taskPaths = taskPaths.slice((taskPaths.length===4?3:4), taskPaths.length);
                const task = tasks[taskName];
                const taskDraft = this.state.taskDrafts.find(taskD => taskD.name === taskName);
                if (taskDraft) {
                    const taskKeys = Object.keys(task);
                    for (const taskKey of taskKeys) {
                        if (taskKey !== 'specifications_template') {
                            task[taskKey] = taskDraft[taskKey];
                        }
                    }
                }
                if (task) {
                    tasksToUpdate[taskName] = taskName;
                    if (taskPaths.includes("station_groups")) {
                        stationGroups[param.name] = $strategyRefs.get(param.refs[0]);
                    }   else {
                        const taskTemplate = _.find(this.taskTemplates, {'name': task.specifications_template.name,
                                                                        'version': task.specifications_template.version || 1});
                        // Get the default Bandpass filter and pass to the editor to for frequency calculation from subband list
                        if (taskTemplate.type_value === 'observation' && task.specifications_doc.station_configuration?.filter) {
                            bandPassFilter = task.specifications_doc.station_configuration?.filter;
                        }   else if (taskTemplate.type_value === 'observation' && taskTemplate.ref_resolved_schema.properties.filter) {
                            bandPassFilter = taskTemplate.ref_resolved_schema.properties.filter.default;
                        }
                        schema.definitions = {...schema.definitions, ...taskTemplate.ref_resolved_schema.definitions};
                        taskPaths.reverse();
                        const paramProp = await ParserUtility.getParamProperty($strategyRefs, taskPaths, taskTemplate.ref_resolved_schema, this.taskFilters);
                        schema.properties[param.name] = _.cloneDeep(paramProp);
                        if (schema.properties[param.name]) {
                            schema.properties[param.name].title = param.name;
                            try { schema.properties[param.name].default = $strategyRefs.get(param.refs[0]); } catch(err) { /* empty */ }
                            paramsOutput[param.name] = schema.properties[param.name].default || ParserUtility.getDefaultParamValue(schema.properties[param.name].type); 
                        }
                    }
                }
            }
            this.setState({observStrategy: observStrategy, paramsSchema: schema, 
                            paramsOutput: paramsOutput, stationGroups: stationGroups,
                            bandPassFilter:bandPassFilter, tasksToUpdate: tasksToUpdate});
            // this.setState({observStrategy: observStrategy, paramsSchema: _.cloneDeep(schema), paramsOutput: paramsOutput, stationGroup: station_group, isDirty: true});

            // Function called to clear the JSON Editor fields and reload with new schema
            if (this.state.editorFunction) {
                this.state.editorFunction();
            }
        }   else {
            this.setState({observStrategy: null});
            this.setEditorOutput(null, []);
        }
    }

    async componentDidMount() {
        const promises = [  ProjectService.getProjectList(false, 'name,url'), 
                            ScheduleService.getObservationStrategies(),
                            TaskService.getTaskTemplates(),
                            ScheduleService.getSchedulingUnitDraftById(this.props.match.params.id),
                            ScheduleService.getTasksDraftBySchedulingUnitId(this.props.match.params.id, true),
                            ScheduleService.getSchedulingConstraintTemplates(),
                            ScheduleService.getStationGroup(),
                            UtilService.getPriorityQueueType(),
                            TaskService.getTaskFilterDefinition('draft')
                        ];
        const permissionById = await AuthUtil.getUserPermissionByModuleId('scheduling_unit_draft', this.props.match.params.id)
        this.setState({permissionById: permissionById})
        Promise.all(promises).then(responses => {
            this.projects = responses[0];
            this.observStrategies = responses[1];
            this.taskTemplates = responses[2];
            this.constraintTemplates = responses[5];
            this.stations = responses[6];
            this.priorityQueueTypes = responses[7];
            this.taskFilters =  responses[8];
            this.schedulingSets = [responses[3].scheduling_set_object]
            responses[3].project = this.schedulingSets[0].project_id;
            let taskOutputPinned = responses[4].data.results[0]?.output_pinned;
            let triStateValue = true;
            responses[4].data.results.forEach( task => {
                if (taskOutputPinned !== task.output_pinned) {
                    triStateValue = null;
                }
            });
            if (triStateValue === true) {
                // All values are same in tasks
                responses[3]['output_pinned'] = taskOutputPinned;
            }   else {
                // if output_pinned values are different in tasks
                responses[3]['output_pinned'] = null;
            }
            
            this.setState({ schedulingUnit: responses[3], taskDrafts: responses[4].data.results,
                            observStrategyVisible: responses[3].observation_strategy_template_id?true:false });
            this.changeStrategy(responses[3].observation_strategy_template_id);
            const targetObservation = responses[4].data.results.find(task => {return task.template.type_value === 'observation' && task.specifications_doc.station_configuration?.station_groups?true:false});
            this.setState({
                stationGroup: targetObservation ? targetObservation.specifications_doc.station_configuration?.station_groups : []
            });
            if (this.state.schedulingUnit.project) {
                this.setState({isLoading: false, schedulingSets: this.schedulingSets});
            }   else {
                this.setState({isLoading: false});
            }
            const suConstraintTemplate = _.find(this.constraintTemplates, {"id": responses[3].scheduling_constraints_template_id});
            this.constraintStrategy(suConstraintTemplate,  this.formatConstraintDocForUI(this.state.schedulingUnit.scheduling_constraints_doc))
        }); 
    }

    /**
     * Format constraint field value for UI
     * @param {*} scheduling_constraints_doc 
     * @returns 
     */
    formatConstraintDocForUI(scheduling_constraints_doc) {
        if (scheduling_constraints_doc) {
            scheduling_constraints_doc.sky.transit_offset.from = (scheduling_constraints_doc.sky.transit_offset.from<0?'-':'')+UnitConversion.getSecsToHHmmss(scheduling_constraints_doc.sky.transit_offset.from);
            scheduling_constraints_doc.sky.transit_offset.to = (scheduling_constraints_doc.sky.transit_offset.to<0?'-':'')+UnitConversion.getSecsToHHmmss(scheduling_constraints_doc.sky.transit_offset.to);
        }
        return scheduling_constraints_doc;
    }

    /**
     * This is the callback method to be passed to the JSON editor. 
     * JEditor will call this function when there is change in the editor.
     * @param {Object} jsonOutput 
     * @param {Array} errors 
     */
    setEditorOutput(jsonOutput, errors) {
        this.paramsOutput = jsonOutput;
        this.validEditor = errors.length === 0;
        this.updateParameters();
        this.setState({ paramsOutput: jsonOutput, 
            validEditor: errors.length === 0,
            validForm: this.validateForm()});
    }

    setEditorOutputConstraint(jsonOutput, errors) {
        let err = [ ...errors ];
        if (jsonOutput.scheduler === 'online') {
            err = err.filter(e => e.path !== 'root.time.at');
        }
        this.constraintParamsOutput = jsonOutput || {};
        this.constraintValidEditor = err.length === 0;
        if  ( !this.state.isDirty && this.state.constraintParamsOutput && !_.isEqual(this.state.constraintParamsOutput, jsonOutput) ) {
            if(!this.state.isDirty) {
                publish('edit-dirty', true);
            }
            this.setState({ constraintParamsOutput: jsonOutput, 
                constraintValidEditor: err.length === 0,
                validForm: this.validateForm(), isDirty: true});
                
        }   else {
            this.setState({ constraintParamsOutput: jsonOutput, 
                constraintValidEditor: err.length === 0,
                validForm: this.validateForm()});
        }
        this.updateConstraintsParams();
    }

    /**
     * This function is mainly added for Unit Tests. If this function is removed Unit Tests will fail.
     */
    validateEditor() {
        return this.validEditor?true:false;
    }
    
    /**
     * Function to set form values to the SU object
     * @param {string} key 
     * @param {object} value 
     */
    setSchedUnitParams(key, value) {
        let schedulingUnit = _.cloneDeep(this.state.schedulingUnit);
        if (key === 'rank') {
            const previousValue = schedulingUnit.rank;
            value = value.replace(/([^0-9.]+)/, "");
            const match = /(\d{0,1})[^.]*((?:\.\d{0,4})?)/g.exec(value);
            const val = match[1] + match[2];
            if (val === '' || (val >= 0 && val <= 1)) {
                schedulingUnit[key] = val === ''? '':Number(val);
            }   else {
                schedulingUnit[key] = previousValue;
            }
        }  else {
            schedulingUnit[key] = value;
        }
        if  ( !this.state.isDirty && !_.isEqual(this.state.schedulingUnit, schedulingUnit) ) {
            if(!this.state.isDirty) {
                publish('edit-dirty', true);
            }
            this.setState({schedulingUnit: schedulingUnit, validForm: this.validateForm(key), validEditor: this.validateEditor(), isDirty: true});
        }   else {
            this.setState({schedulingUnit: schedulingUnit, validForm: this.validateForm(key), validEditor: this.validateEditor()});
        }
       // this.setState({schedulingUnit: schedulingUnit, validForm: this.validateForm(key), validEditor: this.validateEditor()});
        this.validateEditor();
    }

    /**
     * JEditor's function that to be called when parent wants to trigger change in the JSON Editor
     * @param {Function} editorFunction 
     */
    setEditorFunction(editorFunction) {
        this.setState({editorFunction: editorFunction});
    }

    /**
     * Validation function to validate the form or field based on the form rules.
     * If no argument passed for fieldName, validates all fields in the form.
     * @param {string} fieldName 
     */
    validateForm(fieldName) {
        let validForm = false;
        let errors = this.state.errors;
        let validFields = this.state.validFields;
        if (fieldName) {
            delete errors[fieldName];
            delete validFields[fieldName];
            if (this.formRules[fieldName]) {
                const rule = this.formRules[fieldName];
                const fieldValue = this.state.schedulingUnit[fieldName];
                if (rule.required) {
                    if (!fieldValue) {
                        errors[fieldName] = rule.message?rule.message:`${fieldName} is required`;
                    }   else {
                        validFields[fieldName] = true;
                    }
                }
            }
        }   else {
            errors = {};
            validFields = {};
            for (const fieldName in this.formRules) {
                const rule = this.formRules[fieldName];
                const fieldValue = this.state.schedulingUnit[fieldName];
                if (rule.required) {
                    if (!fieldValue) {
                        errors[fieldName] = rule.message?rule.message:`${fieldName} is required`;
                    }   else {
                        validFields[fieldName] = true;
                    }
                }
            }
        }
        this.setState({errors: errors, validFields: validFields});
        if (Object.keys(validFields).length === Object.keys(this.formRules).length) {
            validForm = true;
        }
        return validForm && this.validateStations() && _.sum(_.values(this.state.missingStnErorsofTask))<=0;
    }

    /**
     * Checks if at least one station selected for each station_group parameter and max_nr_missing station value is entered.
     * @returns Boolean
     */
     validateStations() {
        let validStations = false;
        // Check if at least one station selected for all station groups(parameters)
        // Break the loop if any of the required condition check is not satisfied
        for (const stationGroup of _.keys(this.state.stationGroups)) {
            // Check if the selected stations and custom stations are not empty
            if (this.state.selectedStationsOfTask[stationGroup] && this.state.customStationsOfTask[stationGroup]) {
                // Get the stations list for the station group(parameter)
                const stationsList = this.state.selectedStationsOfTask[stationGroup].concat(this.state.customStationsOfTask[stationGroup]);
                //If stationsList is empty then no station group and custom stations are selected
                if (stationsList.length === 0) {
                    validStations = false;
                    break;
                }   else {
                    //If predefined station group(Dutch, Remote, etc.,) is/are selected, validate if max_nr_missing is entered
                    if (this.state.selectedStationsOfTask[stationGroup].length>0) {
                        for (const selectedStations of this.state.selectedStationsOfTask[stationGroup]) {
                            const selectedGroup = this.state.missingStationsofTask[stationGroup][selectedStations]
                            if (selectedGroup) {
                                if (selectedGroup.max_nr_missing === "") {
                                    validStations = false;
                                    break;
                                }   else {
                                    validStations = true;
                                }
                            }   
                        }
                        if (!validStations) {
                            break;
                        }   else {
                            validStations = true;
                        }
                    }
                    //If custom station group(s) is/are selected, validate if max_nr_missing is entered
                    if (this.state.customStationsOfTask[stationGroup].length>0) {
                        for (const customStations of this.state.customStationsOfTask[stationGroup]) {
                            if (customStations.max_nr_missing === "") {
                                validStations = false;
                                break;
                            }   else {
                                validStations = true;
                            }   
                        }
                        if (!validStations) {
                            break;
                        }   else {
                            validStations = true;
                        }
                    }
                }
            }   else {
                validStations = false;
                break;
            }
        }
        return validStations;
    }
    /**
     * Common function validate specification docs
     * @param {String} template Template url
     * @param {Object} updatedSpecificationDoc Object which needs to be validated
     */
     async validateSpecificationsDoc(updatedObservStrategy) {
        if (updatedObservStrategy) {
            const promises = [];
            promises.push(ScheduleService.validateSpecificationsDoc(updatedObservStrategy.scheduling_unit_template, updatedObservStrategy.template))
            const validation_result = await Promise.all(promises);
            let validator = validation_result[0]
            if (validator!==undefined) {
                let validationMessage = validation_result[0]['message'];
                    if(validation_result[0]['valid']) {
                        validationMessage = "Task Parameters specification is valid"
                    }
                if(validation_result.length > 0) {
                    this.setState({validSpecification: validation_result[0]['valid'],
                            validSpecificationMessage: validationMessage});
                }
            }
        }
    }

    async validateSchedulingConstraints(updatedConstraintParmas) {
        if (updatedConstraintParmas) {
            let schedulingConstraintsDoc = updatedConstraintParmas.scheduling_constraints_doc;
            for (const key of _.keys(schedulingConstraintsDoc.time)) {
                if (!schedulingConstraintsDoc.time[key]) {
                    delete schedulingConstraintsDoc.time[key];
                }
            }
            const promises = [];
            promises.push(ScheduleService.validateSpecificationsDoc(updatedConstraintParmas.constraint.url, schedulingConstraintsDoc))
            const validation_result = await Promise.all(promises);
            if(validation_result.length > 0) {
                let validator = validation_result[0];
                if (validator!==undefined) {
                    let validationMessage = validation_result[0]['message'];
                    if(validation_result[0]['valid']) {
                        validationMessage = "Constraints specification is valid"
                    }
                    this.setState({validConstraints: validation_result[0]['valid'],
                        validConstraintMessage: validationMessage});
                }
            }
        }
    }
    
    /**
     * Function to update task parameters 
     */
     async updateParameters() {
        let observStrategy = null;
        let station_groups = [];
        if (this.state.observStrategy) {
            observStrategy = _.cloneDeep(this.state.observStrategy);
            const $refs = await $RefParser.resolve(observStrategy.template);
            observStrategy.template.parameters.forEach(async(param, index) => {
                // $refs.set(observStrategy.template.parameters[index]['refs'][0], this.state.paramsOutput[param.name]);
                let refIndex = 0;
                for (const ref of param.refs) {
                    station_groups = [];
                    if (ref.includes("station_groups")) {
                        for (const selectedGroup of this.state.selectedStationsOfTask[param.name]) {
                            const stnGroup = this.state.missingStationsofTask[param.name][selectedGroup];
                            station_groups.push({stations: stnGroup.stations, max_nr_missing: parseInt(stnGroup.max_nr_missing)});
                        }
                        for (const customGroup of this.state.customStationsOfTask[param.name]) {
                            station_groups.push({stations: customGroup.stations, max_nr_missing: parseInt(customGroup.max_nr_missing)});
                        }
                        $refs.set(observStrategy.template.parameters[index]['refs'][refIndex], station_groups);
                    }   else {
                        $refs.set(observStrategy.template.parameters[index]['refs'][refIndex], this.state.paramsOutput[param.name]);
                    }
                    refIndex++;
                }
            });
        }
        if(this.state.validEditor) {
            this.validateSpecificationsDoc(observStrategy);
        }
        else {
            this.setState({validSpecification: false, validSpecificationMessage: 'Task Parameters specification is not valid' })
        }
        this.setState({updatedObservStrategy: observStrategy});
    }
    
    /**
     * Function to update constraints
     */
    updateConstraintsParams() {
        const constStrategy = _.cloneDeep(this.state.constraintParamsOutput);  
        if(constStrategy.scheduler=== "manual"){
            if(constStrategy.time.at === undefined){
                this.setState({
                    errorDialog:true
                });
                return false;
            }
        }
        if (constStrategy.scheduler === 'online') {
            // For deleting property
            delete constStrategy.time.at;
            }
            if (!constStrategy.time.after) {
            delete constStrategy.time.after;
        }
        if (!constStrategy.time.before) {
            delete constStrategy.time.before;
            }
        UnitConversion.degreeToRadians(constStrategy.sky);
        for (let type in constStrategy.time) {
            if (constStrategy.time[type] && constStrategy.time[type].length) {
                if (typeof constStrategy.time[type] === 'string') {
                    constStrategy.time[type] = `${moment(constStrategy.time[type]).format("YYYY-MM-DDTHH:mm:ss.SSSSS", { trim: false })}Z`;
                } else {
                    constStrategy.time[type].forEach(time => {
                        for (let key in time) {
                            time[key] = `${moment(time[key] ).format("YYYY-MM-DDTHH:mm:ss.SSSSS", { trim: false })}Z`;
                        }
                        
                    })
                }
            }
        }
        if (constStrategy.sky.transit_offset) {
            constStrategy.sky.transit_offset.from = UnitConversion.getHHmmssToSecs(constStrategy.sky.transit_offset.from);
            constStrategy.sky.transit_offset.to = UnitConversion.getHHmmssToSecs(constStrategy.sky.transit_offset.to);
        }
        const const_strategy = {scheduling_constraints_doc: constStrategy, id: this.state.constraintSchema.id, constraint: this.state.constraintSchema};
        if(this.state.constraintValidEditor) {
            this.validateSchedulingConstraints(const_strategy)
        }
        else {
            this.setState({validConstraints: false, validConstraintMessage: 'Constraints specification is not valid' })
        }
        this.setState({updatedConstraintParmas: const_strategy});
    }
    
    
    /**
     * Function to create Scheduling unit
    */
    async saveSchedulingUnit() {
        await this.updateParameters();
        const constStrategy = _.cloneDeep(this.state.updatedConstraintParmas);
        let observStrategy = _.cloneDeep(this.state.updatedObservStrategy);
        const schUnit = { ...this.state.schedulingUnit };
        schUnit.scheduling_constraints_doc = constStrategy;
        let schedulingUnit = this.state.schedulingUnit;
        schUnit.rank = schUnit.rank === ''?0:schUnit.rank.toFixed(4);
        schUnit['scheduling_constraints_doc'] = schUnit.scheduling_constraints_doc.scheduling_constraints_doc;
        schUnit['specifications_doc'] = observStrategy.template;
        schedulingUnit = await ScheduleService.updateSchedulingUnitDraft(schUnit);
        if (!schedulingUnit.error) {
                appGrowl.show({severity: 'success', summary: 'Success', detail: 'Scheduling Unit and Tasks updated successfully!'});
                this.props.history.length>2 ? this.props.history.go(-2):
                    this.props.history.push({
                        pathname: `/schedulingunit/view/draft/${this.props.match.params.id}`,
                    });
        } else {
            appGrowl.show({severity: 'error', summary: 'Error Occured', detail: schedulingUnit.message || 'Unable to Update Scheduling Unit/Tasks'});
        } 
        this.setState({isDirty: false});
        publish('edit-dirty', false);
        
    }
    
    /**
     * warn before cancel the page if any changes detected 
     */
    checkIsDirty() {
        if( this.state.isDirty ){
                this.setState({showDialog: true});
        } else {
            this.cancelCreate();
        }
    }
    
    close() {
        this.setState({showDialog: false});
    }

    /**
     * Cancel SU creation and redirect
     */
    cancelCreate() {
        publish('edit-dirty', false);
        this.props.history.length>1?this.state.isDirty?this.props.history.go(-2):this.props.history.go(-1):this.props.history.push(`/schedulingunit/view/draft/${this.props.match.params.id}`);
        this.setState({showDialog: false});
    }

    async constraintStrategy(schema, initValue){
        this.setState({ constraintSchema: schema, initValue: initValue});
    }
  
    onUpdateStations = (state, selectedStations, missing_StationFieldsErrors, customSelectedStations, taskName) => {
        let selectedStationsOfTask = this.state.selectedStationsOfTask;
        let taskStations = selectedStationsOfTask[taskName] || [];
        let customStationsOfTask = this.state.customStationsOfTask;
        let taskCustomStations = customStationsOfTask[taskName] || [];
        let missingStationsofTask = this.state.missingStationsofTask;
        let missingStnErorsofTask = this.state.missingStnErorsofTask;
        selectedStationsOfTask[taskName] = selectedStations;
        customStationsOfTask[taskName] = customSelectedStations;
        missingStationsofTask[taskName] = state;
        missingStnErorsofTask[taskName] = missing_StationFieldsErrors;
        if  ( !this.state.isDirty ) {
            if (!_.isEqual(taskStations, selectedStations) || !_.isEqual(taskCustomStations, customSelectedStations)) {
                this.setState({missingStationsofTask, selectedStationsOfTask, missingStnErorsofTask, customStationsOfTask }, () => {
                    this.setState({ validForm: this.validateForm(), isDirty: true });
                    if(!this.state.isDirty) {
                        publish('edit-dirty', true);
                    }
                });
            }   else {
                this.setState({missingStationsofTask, selectedStationsOfTask, missingStnErorsofTask, customStationsOfTask }, () => {
                    this.setState({ validForm: this.validateForm() });
                });
            }
        }   else {
            this.setState({missingStationsofTask, selectedStationsOfTask, missingStnErorsofTask, customStationsOfTask }, () => {
                this.setState({ validForm: this.validateForm() });
            });
        }
    };

    render() {
        if (this.state.redirect) {
            return <Redirect to={ {pathname: this.state.redirect} }></Redirect>
        }
        
        const schema = this.state.paramsSchema;
        let jeditor = null;
        if (schema) {
            jeditor = React.createElement(Jeditor, {title: "Task Parameters",
                                                        schema: schema,
                                                        initValue: this.state.paramsOutput, 
                                                        callback: this.setEditorOutput,
                                                        parentFunction: this.setEditorFunction,
                                                        bandPassFilter: this.state.bandPassFilter
                                                    });
        }
       return (
            <React.Fragment>
                {this.state.permissionById[this.props.match.params.id] && this.state.permissionById[this.props.match.params.id].edit?
                <>
                    <Toast ref={el => (this.growl = el)} />
                    <PageHeader location={this.props.location} title={'Scheduling Unit - Edit'} 
                            actions={[{icon: 'fa-window-close', title:'Click to Close Scheduling Unit View', 
                            type: 'button',  actOn: 'click', props:{ callback: this.checkIsDirty }}]}/>
                    { this.state.isLoading ? <AppLoader /> :
                    <>
                    <div>
                        <div className="p-fluid">
                            <div className="p-field p-grid">
                                <label htmlFor="schedUnitName" className="col-lg-2 col-md-2 col-sm-12">Name <span style={{color:'red'}}>*</span></label>
                                <div className="col-lg-3 col-md-3 col-sm-12">
                                    <InputText className={this.state.errors.name ?'input-error':''} id="schedUnitName" data-testid="name" 
                                                tooltip="Enter name of the Scheduling Unit" tooltipOptions={this.tooltipOptions} maxLength="128"
                                                ref={input => {this.nameInput = input;}}
                                                value={this.state.schedulingUnit.name} autoFocus
                                                onChange={(e) => this.setSchedUnitParams('name', e.target.value)}
                                                onBlur={(e) => this.setSchedUnitParams('name', e.target.value)}/>
                                    <label className={this.state.errors.name?"error":"info"}>
                                        {this.state.errors.name ? this.state.errors.name : "Max 128 characters"}
                                    </label>
                                </div>
                                <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                <label htmlFor="description" className="col-lg-2 col-md-2 col-sm-12">Description <span style={{color:'red'}}>*</span></label>
                                <div className="col-lg-3 col-md-3 col-sm-12">
                                    <InputTextarea className={this.state.errors.description ?'input-error':''} rows={3} cols={30} 
                                                tooltip="Longer description of the scheduling unit" tooltipOptions={this.tooltipOptions} maxLength="128"
                                                data-testid="description" value={this.state.schedulingUnit.description} 
                                                onChange={(e) => this.setSchedUnitParams('description', e.target.value)}
                                                onBlur={(e) => this.setSchedUnitParams('description', e.target.value)}/>
                                    <label className={this.state.errors.description ?"error":"info"}>
                                        {this.state.errors.description ? this.state.errors.description : "Max 255 characters"}
                                    </label>
                                </div>
                            </div>
                            <div className="p-field p-grid">
                                <label htmlFor="project" className="col-lg-2 col-md-2 col-sm-12">Project </label>
                                <div className="col-lg-3 col-md-3 col-sm-12" data-testid="project" >
                                    <Dropdown inputId="project" optionLabel="name" optionValue="name" 
                                            tooltip="Project" tooltipOptions={this.tooltipOptions}
                                            value={this.state.schedulingUnit.project} disabled={this.state.schedulingUnit.project?true:false}
                                            options={this.projects} 
                                            placeholder="Select Project" />
                                </div>
                                <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                <label htmlFor="schedSet" className="col-lg-2 col-md-2 col-sm-12">Scheduling Set </label>
                                <div className="col-lg-3 col-md-3 col-sm-12">
                                    <Dropdown data-testid="schedSet" id="schedSet" optionLabel="name" optionValue="id" 
                                            tooltip="Scheduling set of the project" tooltipOptions={this.tooltipOptions}
                                            value={this.state.schedulingUnit.scheduling_set_id} 
                                            options={this.state.schedulingSets} 
                                            disabled={this.state.schedulingUnit.scheduling_set_id?true:false}
                                            placeholder="Select Scheduling Set" />
                                </div>
                            </div>
                            <div className="p-field p-grid">
                                <label htmlFor="rank" className="col-lg-2 col-md-2 col-sm-12">Rank</label>
                                <div className="col-lg-3 col-md-3 col-sm-12" data-testid="rank" >
                                    <input type="number"
                                            data-for="reacttooltip"
                                            data-iscapture="true"
                                            data-tip="Priority of this scheduling unit w.r.t. other scheduling units within the same queue and project.. Min-0.0000, Max-1.0000"
                                            name="rank" data-testid="rank"
                                            className="p-inputtext p-component p-filled"
                                            value={this.state.schedulingUnit.rank}
                                            step="0.0001"
                                            onChange={(e)=> this.setSchedUnitParams('rank', e.target.value)}/>
                                    <label className="error">
                                        {this.state.errors.rank ? this.state.errors.rank : ""}
                                    </label>
                                </div>
                                <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                <label htmlFor="priority_queue" className="col-lg-2 col-md-2 col-sm-12">Priority Queue</label>
                                <div className="col-lg-3 col-md-3 col-sm-10">
                                    <Dropdown data-testid="priority_queue" id="priority_queue" optionLabel="value" optionValue="url" 
                                            tooltip="Priority Queue of the Scheduling Unit" tooltipOptions={this.tooltipOptions}
                                            value={this.state.schedulingUnit.priority_queue} 
                                            options={this.priorityQueueTypes} 
                                            onChange={(e) => {this.setSchedUnitParams('priority_queue',e.value)}} 
                                            placeholder="Select Priority Queue" />
                                    <label className={(this.state.errors.priority_queue && this.state.touched.priority_queue) ?"error":"info"}>
                                        {(this.state.errors.priority_queue && this.state.touched.priority_queue) ? this.state.errors.priority_queue : ""}
                                    </label>
                                </div>
                            </div>
                            <div className="p-field p-grid">
                                { this.state.observStrategyVisible && 
                                    <>
                                        <label htmlFor="observStrategy" className="col-lg-2 col-md-2 col-sm-12">Observation Strategy </label>
                                        <div className="col-lg-3 col-md-3 col-sm-12" data-testid="observStrategy" >
                                            <Dropdown inputId="observStrategy" optionLabel="name" optionValue="id" 
                                                    tooltip="Observation Strategy Template to be used to create the Scheduling Unit and Tasks" tooltipOptions={this.tooltipOptions}
                                                    value={this.state.schedulingUnit.observation_strategy_template_id} 
                                                    disabled={this.state.schedulingUnit.observation_strategy_template_id?true:false} 
                                                    options={this.observStrategies} 
                                                    onChange={(e) => {this.changeStrategy(e)}} 
                                                    placeholder="Select Strategy" />
                                            <label className="info">
                                                {this.state.observStrategy? this.state.observStrategy.description : "Select Observation Strategy"}
                                            </label>
                                        </div>
                                        <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    </>
                                }
                                
                                    <label htmlFor="project" className="col-lg-2 col-md-2 col-sm-12">Pin Data</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12" data-testid="project" >
                                        <TriStateCheckbox  inputId="trigger" role="trigger" 
                                            onChange={(e) => {this.setSchedUnitParams('output_pinned', e.value)}}  
                                            value={this.state.schedulingUnit.output_pinned}
                                            tooltip="Do not delete data automatically after successful ingest" 
                                            tooltipOptions={this.tooltipOptions}
                                        ></TriStateCheckbox>
                                    </div>
                            </div>
                        </div>
                        {_.keys(this.state.stationGroups).length >0 &&
                        <div className='grouping'>
                            <fieldset>
                                <h3 className="card-title level-1 je-object__title" style={{display: "inline-block"}}>
                                    <button type="button" title={this.state.stationsCollapsed?"Expand":"Collapse"} 
                                        className="btn btn-secondary btn-sm json-editor-btn-collapse json-editor-btntype-toggle" 
                                        onClick={() => this.setState({stationsCollapsed: !this.state.stationsCollapsed})}>
                                        <i className={this.state.stationsCollapsed?"fas fa-caret-right":"fas fa-caret-down"}></i>
                                    </button>
                                    <label>Station specification</label>
                                </h3>
                                {!this.state.stationsCollapsed &&
                                <>
                                    { _.keys(this.state.stationGroups).map((stnGroup) => {
                                        return <Stations
                                        taskName={stnGroup}
                                        stationGroup={this.state.stationGroups[stnGroup]}
                                        onUpdateStations={this.onUpdateStations.bind(this)}
                                        height={'auto'}
                                        key={stnGroup}
                                    />
                                    })}
                                </>
                                }
                            </fieldset>
                        </div>
                        }

                        {this.state.constraintSchema && <div className="p-fluid">
                            <fieldset className="border-style">
                                <div className="p-col-12" style={{width: "100%"}}>
                                <SchedulingConstraint initValue={this.state.initValue} constraintTemplate={this.state.constraintSchema} callback={this.setEditorOutputConstraint} />
                                </div>
                            </fieldset>
                            <div className="p-col-12" style={{marginLeft: '0.2em', marginTop: '-10px', paddingTop:'0px'}}>
                                <label className={this.state.validConstraints ? "info" : "error"}>
                                    {this.state.validConstraintMessage ? this.state.validConstraintMessage : ""}
                                </label>
                            </div>
                        </div>}
                        
                        {this.state.paramsSchema &&
                            <div className="p-fluid">
                                <fieldset className="border-style">
                                    <div className="p-col-12" style={{marginLeft: '-0.5em', marginTop: '-10px', paddingTop:'0px'}}>
                                        {jeditor}
                                    </div> 
                                </fieldset>
                                <div className="p-col-12" style={{marginLeft: '0.2em', marginBottom: "2em"}}>
                                    <label className={this.state.validSpecification ? "info" : "error"}>
                                        {this.state.validSpecificationMessage ? this.state.validSpecificationMessage : ""}
                                    </label>
                                </div>
                            </div>   
                        }
                        <ReactTooltip id="reacttooltip" place={'left'}  type={'dark'} effect={'solid'} multiline={true} />
                        <div className="p-breadcrumb footer-long-div actions" >
                            <FormActionbar onSubmit={this.saveSchedulingUnit} 
                            disableSaveBtn={!this.state.constraintValidEditor || !this.state.validEditor || !this.state.validForm || !this.state.validConstraints || !this.state.validSpecification} onCancel={this.checkIsDirty} />
                        </div>

                        <div className="p-grid" data-testid="confirm_dialog">
                            <CustomDialog type="confirmation" visible={this.state.showDialog} width="40vw"
                                header={'Edit Scheduling Unit'} message={'Do you want to discard your changes? Your changes to the scheduling unit will not be saved.'} 
                                content={''} onClose={this.close} 
                                actions={ [ {id:"yes", title: 'Discard', callback: this.cancelCreate, className:'act-btn-dispose'},
                                    {id:"no", title: 'Cancel', className:'act-btn-cancel', callback: this.close} ]}                            
                                >        
                            </CustomDialog>
                        </div>
                    </div>
                           <CustomDialog type="confirmation" visible={this.state.errorDialog} width="40vw"
                                header={'Error'} message={'Do you want to discard your changes? Your changes to the scheduling unit will not be saved.'} 
                                content={''} onClose={this.close}
                                actions={ [ {id:"yes", title: 'Discard', callback: this.cancelCreate, className:'act-btn-dispose' },
                                    {id:"no", title: 'Cancel', className:'act-btn-cancel', callback: this.close} ]}                            
                                >        
                            </CustomDialog>
                    </>
                    }
                </>:<AccessDenied/>}
                <CustomDialog type="warning" visible={this.state.errorDialog} width="40vw"
                        header={'Alert'} message={'Start at this moment cannot be empty'} 
                        content={''}  onClose={()=>this.setState({ errorDialog: false })}>
                    </CustomDialog>
            </React.Fragment>
        );
    }
}
export default EditSchedulingUnit