import React, { Component } from 'react';
import { Link } from 'react-router-dom/cjs/react-router-dom.min';
import moment from 'moment';
import _ from 'lodash';
import ViewTable from '../../components/ViewTable';
import { JsonToTable } from "react-json-to-table";
import SchedulingConstraints from './Scheduling.Constraints';
import UIConstants from '../../utils/ui.constants';
import { Dialog } from "primereact/dialog";
import { Button } from "primereact/button";
import AuthStore from '../../authenticate/auth.store';
import AuthUtil from '../../utils/auth.util';
import { CustomDialog } from '../../layout/components/CustomDialog';
import UnitConverter from '../../utils/unit.converter';
import ScheduleService from '../../services/schedule.service';
import { appGrowl } from '../../layout/components/AppGrowl';
import DetailsSummary from "../Timeline/components/DetailsSummary";
import { Panel } from 'primereact/panel';
/**
 * Component to view summary of the scheduling unit with limited task details
 */
export class SchedulingUnitSummary extends Component {
    constructor(props) {
        super(props);
        this.state = {
            schedulingUnit: props.schedulingUnit || null,
            updatedConstraintsValue: {},
            isEdit: false,
            showerror: 0,
            editCheck: 1,
            close: false,
            userrole: AuthStore.getState(),
            constraintsTemplate: this.props.constraintsTemplate,
            dialog: { header: 'Confirm', detail: '' },
            dialogVisible: false,
        };
        this.currentSU = {};
        this.access_denied_message = "Don't have permission";
        this.constraintsOrder = ['scheduler', 'time', 'daily', 'sky'];
        this.closeSUDets = this.closeSUDets.bind(this);
        this.setConstraintsEditorOutput = this.setConstraintsEditorOutput.bind(this);
        this.checkIsDirty = this.checkIsDirty.bind(this);
        this.confirmAutoDeletion = this.confirmAutoDeletion.bind(this);
        this.setAutoDeletion = this.setAutoDeletion.bind(this);
        this.showCancelSUConfirmation = this.showCancelSUConfirmation.bind(this);
        this.cancelSchedulingUnit = this.cancelSchedulingUnit.bind(this);
        this.closeDialog = this.closeDialog.bind(this);

    }


     getHeaderTemplate = (permissions) => {
        return <div className="p-panel-header justify-content-start">
               <span className="p-panel-title" >Scheduling Unit Details</span>
            <Link onClick={this.redirectToSUDetails} title="View Full Details">
                <i className="fa fa-eye" style={{ paddingRight: '3px' }}></i></Link>
            <Link onClick={this.showCancelSUConfirmation}
                title={this.state.schedulingUnit.status === 'cancelled' ? "Scheduling Unit already cancelled" : "Cancel Scheduling Unit"}
                className={this.state.schedulingUnit.status === 'cancelled' ? "act-btn-disable" : ""} >
                <i className="fa fa-ban"
                    style={{
                        fontSize: '16px', paddingRight: '3px', color: this.state.schedulingUnit.status === 'cancelled' ? 'gray' : '',
                        cursor: this.state.schedulingUnit.status === 'cancelled' ? 'not-allowed' : ''
                    }}></i></Link>
            <Link onClick={this.confirmAutoDeletion}
                title={permissions.autodeletion ? this.state.schedulingUnit.output_pinned ? 'Unpin data after ingest' : 'Pin data after ingest' : `${this.access_denied_message} to do Pin/Unpin data after ingest`}>
                <i className="fa fa-thumbtack" style={{ fontSize: '16px', paddingRight: '3px' }}></i></Link>
            <Link to={this.props.location ? this.props.location.pathname : "/su/timelineview/week"} onClick={this.closeSUDets} title="Close Details"><i className="pi pi-times"></i></Link>
        </div>
    }

    /**
     * Function to close the summary panel and call parent callback function to close.
     */
    closeSUDets() {
        if (this.props.closeCallback) {
            this.props.closeCallback();
        }
    }


    handleKeyDownForClose = (e) => {
        if (e.key === 'Enter') {
            this.props.onCloseFn(true)
        }
    }


    /**
     * Order the properties in the constraint object in the predefined order
     * @param {Object} constraintsDoc
     */
    getOrderedConstraints(constraintsDoc, constraintsOrder) {
        let orderedConstraints = {};
        for (const constraintKey of constraintsOrder) {
            /* Format the object to remove empty values*/
            const constraint = this.getFormattedConstraint(constraintsDoc[constraintKey]);
            if (constraint) {
                orderedConstraints[constraintKey.replace('_', ' ')] = constraint;
            }
        }
        return orderedConstraints;
    }

    /**
     * warn before cancel the page if any changes detected
    */
    checkIsDirty() {
        if (this.state.isDirty) {
            this.setState({
                showDialog: true
            });
        } else {
            this.cancelCreate();
        }
    }

    /**
     * Format the constraint object i.e removes the empty values to show only available values.
     * @param {Object} constraint
     */
    getFormattedConstraint(constraint) {
        if (constraint) {
            const objectType = typeof constraint;
            switch (objectType) {
                case "number": {
                    if ((constraint + "").indexOf(".") >= 0) {
                        constraint = ((constraint * 180) / Math.PI).toFixed(2);
                    }
                    break;
                }
                case "string": {
                    try {
                        const dateConstraint = moment.utc(constraint);
                        if (dateConstraint.isValid()) {
                            constraint = dateConstraint.format(UIConstants.CALENDAR_DATETIME_FORMAT);
                        }
                    } catch (error) { /* empty */ }
                    break;
                }
                case "boolean": {
                    constraint = constraint ? 'Yes' : null;
                    break;
                }
                case "object": {
                    if (Array.isArray(constraint)) {
                        let newArray = []
                        for (let arrayObj of constraint) {
                            arrayObj = this.getFormattedConstraint(arrayObj);
                            if (arrayObj) {
                                newArray.push(arrayObj);
                            }
                        }
                        constraint = newArray.length > 0 ? newArray : null;
                    } else {
                        let newObject = {};
                        let keys = _.keys(constraint);
                        if (keys.indexOf('from') >= 0 & keys.indexOf('to') >= 0) {
                            constraint = this.getOrderedConstraints(constraint, ['from', 'to']);
                        }
                        for (const objectKey of _.keys(constraint)) {
                            if (objectKey === 'reference_pointing') {
                                let refPointing = constraint[objectKey];
                                refPointing['pointing']['angle1'] = UnitConverter.getAngleInput(refPointing['pointing']['angle1']);
                                refPointing['pointing']['angle2'] = UnitConverter.getAngleInput(refPointing['pointing']['angle2'], true);
                                refPointing['enabled'] = refPointing.enabled ? 'true' : 'false';
                                newObject[objectKey.replace(/_/g, ' ')] = refPointing;
                            } else {
                                let object = this.getFormattedConstraint(constraint[objectKey]);
                                if (object) {
                                    if (_.includes(['sun', 'moon', 'jupiter', 'target', 'calibrator'], objectKey)) {
                                        newObject[(objectKey.replace(/_/g, ' ')) + ' (Degrees)'] = object;
                                    } else {
                                        newObject[objectKey.replace(/_/g, ' ')] = object;
                                    }
                                }
                            }
                        }
                        constraint = (!_.isEmpty(newObject)) ? newObject : null;
                    }
                    break;
                }
                default: { /* empty */ }
            }
        }
        return constraint;
    }

    /**
     * Gets the output from the SchedulingConstraints editor without output formatting so that the values entered in the
     * editor can be shown in the summary without any conversion.
     * @param {Object} jsonOutput
    */
    setConstraintsEditorOutput(jsonOutput, errors) {
        /* to update particular value on current component */
        this.setState({
            updatedConstraintsValue: jsonOutput,
            showerror: errors.length,
            editCheck: this.state.editCheck + 1
        },
            /* to update parent component state value */
            this.props.newPopupvalue(jsonOutput))
        return false;
    }

    redirectToSUDetails = () => {
        if (!this.props.viewInNewWindow) {
            this.props.history.push(`/schedulingunit/view/blueprint/${this.props.schedulingUnit.id}`);
        } else {
            window.open(`/schedulingunit/view/blueprint/${this.props.schedulingUnit.id}`, '_blank');
        }
    }

    async componentDidMount() {
        const promises = [
            AuthUtil.getUserRolePermission()
        ];
        this.setState({ disableCancelBtn: this.props.schedulingUnit.status === 'cancelled' ? true : false });
        this.updateConstraintTemplate();
        await Promise.all(promises).then(responses => {
            this.setState({ userrole: responses[0], editCheck: 1 });
        });
    }

    /**
     * Add new fields for plus/minu value for offset (from & to)
     * @param {*} constraintTemplate
     * @returns
     */
    async updateConstraintTemplate() {
        if (this.props.constraintsTemplate) {
            let constraintTemplate = this.props.constraintsTemplate;
            constraintTemplate.schema.properties.sky.properties.transit_offset.properties.from.default =
                UnitConverter.getSecsToHHmmssWithSign(this.props.constraintsTemplate.schema.properties.sky.properties.transit_offset.properties.from.default);
            constraintTemplate.schema.properties.sky.properties.transit_offset.properties.to.default =
                UnitConverter.getSecsToHHmmssWithSign(this.props.constraintsTemplate.schema.properties.sky.properties.transit_offset.properties.to.default);
            this.setState({ constraintTemplate: constraintTemplate });
        }
    }

    /**
     * Format constraint field value for UI
     * @param {*} scheduling_constraints_doc
     * @returns
     */
    formatConstraintDocForUI(scheduleunit) {
        if (scheduleunit.scheduling_constraints_doc && scheduleunit.scheduling_constraints_doc.sky && scheduleunit.scheduling_constraints_doc.sky.transit_offset) {
            scheduleunit.scheduling_constraints_doc.sky.transit_offset.from = UnitConverter.getSecsToHHmmssWithSign(scheduleunit.scheduling_constraints_doc.sky.transit_offset.from);
            scheduleunit.scheduling_constraints_doc.sky.transit_offset.to = UnitConverter.getSecsToHHmmssWithSign(scheduleunit.scheduling_constraints_doc.sky.transit_offset.to);
        }
        if (this.props.schedulingUnit?.scheduling_constraints_doc?.sky?.transit_offset) {
            this.props.schedulingUnit.scheduling_constraints_doc.sky.transit_offset.from = UnitConverter.getSecsToHHmmssWithSign(this.props.schedulingUnit.scheduling_constraints_doc.sky.transit_offset.from);
            this.props.schedulingUnit.scheduling_constraints_doc.sky.transit_offset.to = UnitConverter.getSecsToHHmmssWithSign(this.props.schedulingUnit.scheduling_constraints_doc.sky.transit_offset.to);
        }
        return scheduleunit;
    }

    /**
      * Confirm Autodeletion before update the value
      */
    confirmAutoDeletion() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Confirm Pin/Unpin data after ingest";
        dialog.detail = this.props.schedulingUnit.output_pinned ? 'Do you want to Unpin data after ingest?' : 'Do you want to Pin data after ingest?';
        dialog.content = '';
        dialog.actions = [{
            id: 'yes', title: this.props.schedulingUnit.output_pinned ? 'Unpin data' : 'Pin data',
            className: this.props.schedulingUnit.output_pinned ? 'act-btn-dispose' : '', callback: this.setAutoDeletion
        },
        { id: 'no', title: 'Cancel', className: 'act-btn-cancel', callback: this.closeDialog }];
        dialog.onSubmit = this.setAutoDeletion;
        dialog.width = '35vw';
        dialog.showIcon = false;
        this.setState({ dialog: dialog, dialogVisible: true });
    }

    /**
     * Close the dialog prompted.
     */
    closeDialog() {
        this.setState({ dialogVisible: false });
    }

    /**
      * Enable/Disable autodeletion in the scheduling unit
      */
    async setAutoDeletion() {
        let suCopy = _.cloneDeep(this.props.schedulingUnit);
        let tasks = [];
        let updatedResSU = await ScheduleService.updateSchedulingUnitValue('blueprint',
            { id: this.props.schedulingUnit.id, 'output_pinned': !this.props.schedulingUnit.output_pinned }, tasks);
        if (updatedResSU) {
            appGrowl.show({ severity: 'success', summary: 'Success', detail: `${updatedResSU.output_pinned ? 'Pin' : 'Unpin'} data after ingest updated successfully` });
            suCopy['output_pinned'] = updatedResSU.output_pinned;
            this.setState({ schedulingUnit: suCopy, dialogVisible: false, });
        } else {
            appGrowl.show({ severity: 'error', summary: 'Failed', detail: 'Unable to update Automatic Deletion' });
            this.setState({ dialogVisible: false });
        }
    }

    /**
     * Show confirmation dialog before cancelling the scheduling unit.
     */
    showCancelSUConfirmation() {
        if (this.state.schedulingUnit.status === 'cancelled') {
            return;
        }
        let dialog = this.state.dialog;
        dialog.detail = '';
        dialog.type = "confirmation";
        dialog.header = "Confirm to Cancel Scheduling Unit";
        dialog.actions = [{ id: 'yes', title: 'Cancel SU', callback: this.cancelSchedulingUnit, style: { "background": "#e91224", "border": "1px solid #e91224" } },
        { id: 'no', title: 'No', className: 'act-btn-cancel', callback: this.closeDialog }];
        if (UIConstants.SU_NOT_STARTED_STATUSES.indexOf(this.props.schedulingUnit.status) >= 0) {
            dialog.detail = "Cancelling this scheduling unit means it will no longer be executed. This action cannot be undone. Do you want to proceed?";
        } else if (UIConstants.SU_ACTIVE_STATUSES.indexOf(this.props.schedulingUnit.status) >= 0) {
            dialog.detail = "Cancelling this scheduling unit means it will be aborted. This action cannot be undone. Do you want to proceed?";
        }
        dialog.submit = this.cancelSchedulingUnit;
        dialog.width = '40vw';
        dialog.showIcon = true;
        this.setState({ dialog: dialog, dialogVisible: true });
    }

    /**
     * Function to cancel the scheduling unit.
     */
    async cancelSchedulingUnit() {
        let cancelledSU = await ScheduleService.cancelSchedulingUnit(this.props.schedulingUnit.id);
        if (!cancelledSU) {
            appGrowl.show({ severity: 'error', summary: 'error', detail: 'Error while cancelling Scheduling Unit' });
            this.setState({ dialogVisible: false });
        } else {
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Scheduling Unit is cancelled successfully' });
            this.setState({ schedulingUnit: cancelledSU, dialogVisible: false });
        }
    }

    

render() {
    const permissions = this.state.userrole.userRolePermission.scheduleunit;
    let schedulingUnit = _.cloneDeep(this.props.schedulingUnit);
    if (this.currentSU.id !== schedulingUnit.id) {
        this.setState({ schedulingUnit: schedulingUnit });
        this.currentSU = schedulingUnit;
    }

    schedulingUnit.task_blueprints?.map(task => {
        task.typeValue = task.specifications_template.type_value;
        task.obsolete = task.obsolete_since ? true : false;
        task["actionpath"] = `/task/view/blueprint/${task.id}/`;
        if (task.typeValue === 'observation') {
            task['links'] = ['Control ID'];
            task['linksURL'] = {
                'Control ID': `${process.env.REACT_APP_SERVER_NAME}/inspect/HTML/${task.controlId}/`
            };
        }
        return task;
    });
    const constraintsTemplate = this.props.constraintsTemplate;
    // After receiving output from the SchedulingConstraint editor order and format it to display
    schedulingUnit = this.formatConstraintDocForUI(schedulingUnit);
    let constraintsDoc = schedulingUnit.scheduling_constraints_doc ? this.getOrderedConstraints(schedulingUnit.scheduling_constraints_doc, this.constraintsOrder) : null;
    let disableBtn = this.state.showerror !== 0 ? true : false;

    let summaryFields = [
        { labelName: "Name", value: schedulingUnit.name, formatTime: false },
        { labelName: "Scheduling Unit Id", value: schedulingUnit.id, formatTime: false },
        { labelName: "Project", value: schedulingUnit.project, formatTime: false },
        { labelName: "Process Start Time", value: schedulingUnit.actual_process_start_time ? schedulingUnit.actual_process_start_time : schedulingUnit.process_start_time, formatTime: true },
        { labelName: "Process End Time", value: schedulingUnit.actual_process_stop_time ? schedulingUnit.actual_process_stop_time : schedulingUnit.process_stop_time, formatTime: true },
        { labelName: "On Sky Start Time", value: schedulingUnit.actual_on_sky_start_time ? schedulingUnit.actual_on_sky_start_time : schedulingUnit.on_sky_start_time, formatTime: true },
        { labelName: "On Sky End Time", value: schedulingUnit.actual_on_sky_stop_time ? schedulingUnit.actual_on_sky_stop_time : schedulingUnit.on_sky_stop_time, formatTime: true },
        { labelName: "Status", value: schedulingUnit.status, formatTime: false }]

    if (schedulingUnit.status === 'unschedulable') {
        summaryFields.push({ labelName: "Unschedulable Reason", value: schedulingUnit.unschedulable_reason, formatTime: false })
    }

    return (
        <React.Fragment>
            {schedulingUnit &&
                <Panel header="Scheduling Unit Details" headerTemplate={this.getHeaderTemplate(permissions)} >

                    <DetailsSummary fields={summaryFields} />
                    {constraintsDoc &&
                            <>
                                <div className="col-12 constraints-summary" style={{ top: "0px", position: "relative", }}>
                                    <label>Constraints:</label>
                                    {permissions.edit && (_.includes(_.split(process.env.REACT_APP_CONSTRAINT_EDITABLE_STATUS, ','), schedulingUnit.status)) ?
                                        <i className="fa fa-pencil-alt" style={{ marginLeft: "5px", cursor: "pointer" }}
                                            title={'Edit Scheduling Constraint'}
                                            onClick={() => this.props.onCloseFn(true)}
                                            onKeyDown={this.handleKeyDownForClose}
                                        >
                                        </i> :
                                        <i className="fa fa-pencil-alt" style={{ marginLeft: "5px", color: "grey", cursor: "not-allowed" }}
                                            title={permissions.edit ? `Constraints of scheduling unit with status ${schedulingUnit.status} cannot be edited` :
                                                `Don't have permission to edit constraints`}
                                        ></i>
                                    }
                                </div>
                                <div className="col-12">
                                    <JsonToTable json={constraintsDoc} />
                                </div>
                            </>
                    }

                    <div className="station-information">
                        <div>
                            <div className="col-12"><label>Stations:</label></div>
                            <div className="col-12 station-list">
                                {schedulingUnit.stations?.map((station ) => (
                                    <div key={`stn-${station}`}>{station}</div>
                                ))}
                            </div>
                        </div>
                        <div>
                            {schedulingUnit.missingStations && schedulingUnit.missingStations.length > 0 &&
                                <div className="col-12"><label>Missing Stations:</label>
                                    <div className="col-12 station-list">
                                        {schedulingUnit.missingStations?.map((station ) => (
                                            <div key={`mis-stn-${station}`}>{station}</div>
                                        ))}
                                    </div>
                                </div>
                            }
                        </div>
                    </div>


                    <div className="col-12 task-summary">
                        <label>Tasks:</label>
                        {schedulingUnit.task_blueprints &&
                        <ViewTable
                            data={schedulingUnit.task_blueprints}
                            defaultcolumns={[{
                                id: "ID", name: "Name", short_description: "Short Description",
                                typeValue: "Type", subtasks_ids: 'Control ID',
                                scheduled_start_time: { name: "Start Time", format: UIConstants.CALENDAR_DATETIME_FORMAT },
                                scheduled_stop_time: { name: "End Time", format: UIConstants.CALENDAR_DATETIME_FORMAT },
                                status: { name: "Status", showTag: true, tagClass: 'su-$dynamic$' }, unschedulable_reason: { name: "Unschedulable Reason" }, obsolete: 'Obsolete'
                            }]}
                            optionalcolumns={[{ actionpath: "actionpath" }]}
                            columnclassname={[{
                                "ID": "filter-input-50", "Name": "filter-input-75", "Type": "filter-input-75", "Control ID": "filter-input-75", "Start Time": "filter-input-75", "End Time": "filter-input-75",
                                "Status": "filter-input-75", "Antenna Set": "filter-input-75", "Band": "filter-input-75"
                            }]}
                            defaultSortColumn={[{ id: "ID", desc: false }]}
                            showaction="false"
                            tablename="timeline_su_taskslist"
                            showTopTotal={false}
                            showGlobalFilter={false}
                            showColumnFilter={false}
                            allowColumnSelection={false}
                        />}
                        {!schedulingUnit.task_blueprints &&
                        <h1>Error retrieving data</h1>
                }
                    </div>
                </Panel>
            }

            {/* model for Constraints start */}
            <div className={`custom-dlg`}>
                <Dialog id="edit_constraint"
                    maximizable maximized={false}
                    style={{ width: '80vw' }}
                    header="Scheduling Constraints - Edit"
                    visible={this.props.onClose || false}
                    onHide={
                        () => {
                            if (this.state.editCheck < 3) {
                                this.setState({
                                    editCheck: 1
                                }, this.props.onCloseFn(false));
                            } else {
                                if (this.props.onSavedisable === false)
                                    this.setState({
                                        editCheck: 1
                                    }, this.props.cancelCheckstatus(true));
                            }
                        }
                    }
                    modal={true}
                    footer={
                        <div>
                            <Button key="submit" className="p-button-primary"
                                disabled={disableBtn}
                                icon="pi pi-check" label="Save" onClick={() => {
                                    this.props.onUpdate(this.props.schedulingUnit.id, this.state.updatedConstraintsValue)
                                    this.setState({
                                        editCheck: 1
                                    });
                                }
                                }
                            />
                            <Button key="cancel" className="act-btn-cancel" icon="pi pi-times" label="Cancel" onClick={() => {
                                if (this.state.editCheck < 3) {
                                    this.setState({
                                        editCheck: 1
                                    }, this.props.onCloseFn(false));
                                } else {
                                    if (this.props.onSavedisable === false)
                                        this.setState({
                                            editCheck: 1
                                        }, this.props.cancelCheckstatus(true));
                                }
                            }}
                            />
                        </div>
                    }
                >
                    <SchedulingConstraints
                        constraintTemplate={constraintsTemplate}
                        initValue={this.props.schedulingUnit.scheduling_constraints_doc}
                        callback={this.setConstraintsEditorOutput}
                    />
                </Dialog>
            </div>
            {/* moder for Constraints End */}

            <CustomDialog type="confirmation" visible={this.props.cancelStatus} width="40vw"
                header={'Edit Scheduling Constraints'} message={'Do you want to discard your changes? Your changes to the Constraints will not be saved.'}
                content={''}
                onClose={
                    () => {
                        if (this.state.editCheck < 3 && this.state.close === true) {
                            this.setState({
                                editCheck: 1,
                                close: false
                            }, this.props.cancelCheckstatus(false));
                        } else if (this.state.editCheck < 3 && this.state.close === false) {
                            this.setState({
                                editCheck: 3
                            }, this.props.cancelCheckstatus(false));
                        } else {
                            this.props.cancelCheckstatus(false)
                        }
                    }
                }

                actions={[{
                    id: "yes", title: 'Discard', callback: () => {
                        this.setState({
                            editCheck: 1,
                            close: true
                        }, this.props.onCloseFn(false))
                        return false;
                    }, className: 'act-btn-dispose'
                },
                {
                    id: "no", title: 'Cancel', className: 'act-btn-cancel', callback: () => {
                        if (this.state.editCheck < 3 && this.state.close === true) {
                            this.setState({
                                editCheck: 1,
                                close: false
                            }, this.props.cancelCheckstatus(false));
                        } else if (this.state.editCheck < 3 && this.state.close === false) {
                            this.setState({
                                editCheck: 3
                            }, this.props.cancelCheckstatus(false));
                        } else {
                            this.props.cancelCheckstatus(false)
                        }
                    }
                }]}
            >
            </CustomDialog>
            <CustomDialog type="confirmation" visible={this.state.dialogVisible}
                header={this.state.dialog.header} message={this.state.dialog.detail} actions={this.state.dialog.actions}
                content={this.state.dialog.content} width={this.state.dialog.width} showIcon={this.state.dialog.showIcon}
                onClose={this.closeDialog} onCancel={this.closeDialog} onSubmit={this.state.dialog.onSubmit} />
        </React.Fragment>
    );
}
}

export default SchedulingUnitSummary;