import moment from "moment";
import UnitConverter from "../../../utils/unit.converter";

export function expandTimeLineRowsForGrouping(timeLineRows, groupingkey) {
    const includeStationsInfo = groupingkey == "station_group"
    const includeStationsUsedInfo = groupingkey == "station_used_group"
    const includeStationsMissingInfo = groupingkey == "station_missing_group"
    const includeStationsgroupInfo = groupingkey == "stationgroup_group"

    const includeTasksInfo = groupingkey == "project_task_group"
    let expandedschedulingUnitItems = [];

    if (includeStationsgroupInfo) expandTimeLineForStationGroup(timeLineRows, expandedschedulingUnitItems);
    if (includeStationsInfo) expandTimeLineForStations(timeLineRows, expandedschedulingUnitItems, "specified_stations", groupingkey);
    if (includeStationsUsedInfo) expandTimeLineForStations(timeLineRows, expandedschedulingUnitItems, "used_stations", groupingkey);
    if (includeStationsMissingInfo) expandTimeLineForStations(timeLineRows, expandedschedulingUnitItems, "missing_stations", groupingkey);
    if (includeTasksInfo) expandTimeLineForProjectTask(timeLineRows, expandedschedulingUnitItems);

    return expandedschedulingUnitItems;
}

function expandTimeLineForStations(timeLineRows, expandedschedulingUnitItems, propertykey, groupingkey) {
    for (const timelinerow of timeLineRows) {
        let stationgroups = timelinerow[propertykey];
        if (stationgroups) {
            for (const station of stationgroups) {
                generateNewStationTimeLineRow(timelinerow, expandedschedulingUnitItems, station, groupingkey);
            }
        }
    }

}

// we split up the CS/RS/IS Stations into possible 3 rows.
function expandTimeLineForStationGroup(timeLineRows, expandedschedulingUnitItems) {

    for (const timelinerow of timeLineRows) {
        const stationsgroups = timelinerow.stationsgroups;
        if (!stationsgroups) continue;
        if (stationsgroups.CS > 0) {
            generateNewStationGroupTimeLineRow(timelinerow, expandedschedulingUnitItems, "CS");
        }
        if (stationsgroups.RS > 0) {
            generateNewStationGroupTimeLineRow(timelinerow, expandedschedulingUnitItems, "RS");
        }
        if (stationsgroups.IS > 0) {
            generateNewStationGroupTimeLineRow(timelinerow, expandedschedulingUnitItems, "IS");
        }
        delete timelinerow.stationsgroups;
    }
}

function expandTimeLineForProjectTask(timeLineRows, expandedschedulingUnitItems) {

    for (const timelinerow of timeLineRows) {
        let tasks = timelinerow.task_blueprints;
        if (tasks) {
            for (const task of tasks) {
                generateNewProjectTaskTimeLineRow(timelinerow, expandedschedulingUnitItems, task, "project_task_group");
            }
        }
    }


}

function generateNewProjectTaskTimeLineRow(schedulingunitTimelinerow, expandedschedulingUnitItems, task, groupingkey) {
    let newtimelineItem = { ...schedulingunitTimelinerow }
    newtimelineItem[groupingkey] = schedulingunitTimelinerow.project + newtimelineItem.groupingSuffix + "#tasks";
    newtimelineItem.name = newtimelineItem.name + " - " + task.name
    newtimelineItem.id = newtimelineItem.id + newtimelineItem[groupingkey] + task.id;
    newtimelineItem.task = task;
    const duration = `${(UnitConverter.getSecsToHrsWithFractionDigits(task.duration))}Hrs`;

    const startTimeWithDisplayDate = moment(`${newtimelineItem.displayDate.format('YYYY-MM-DD')} ${task.process_start_time?.split('T')[1]}`);
    const endTimeWithDisplayDate = moment(`${newtimelineItem.displayDate.format('YYYY-MM-DD')} ${task.process_stop_time?.split('T')[1]}`)
    const onSkyStartTimeWithDisplayDate = moment(`${newtimelineItem.displayDate.format('YYYY-MM-DD')} ${task.on_sky_start_time?.split('T')[1]}`);
    const onSkyEndTimeWithDisplayDate = moment(`${newtimelineItem.displayDate.format('YYYY-MM-DD')} ${task.on_sky_stop_time?.split('T')[1]}`)

    newtimelineItem.start_time = newtimelineItem.SkyTimesOnTimes ? onSkyStartTimeWithDisplayDate : startTimeWithDisplayDate;
    newtimelineItem.end_time = newtimelineItem.SkyTimesOnTimes ? onSkyEndTimeWithDisplayDate : endTimeWithDisplayDate;
    newtimelineItem.duration = duration;
    newtimelineItem.desc = task.description;


    cleanSingleTimelineGroups(newtimelineItem);
    expandedschedulingUnitItems.push(newtimelineItem);
}

function generateNewStationGroupTimeLineRow(schedulingunitTimelinerow, expandedschedulingUnitItems, stationPrefix) {

    let newtimelineItem = { ...schedulingunitTimelinerow }


    newtimelineItem.stationgroup_group = stationPrefix + newtimelineItem.groupingSuffix;
    newtimelineItem.id = newtimelineItem.id + newtimelineItem.stationgroup_group;
    cleanSingleTimelineGroups(newtimelineItem);
    expandedschedulingUnitItems.push(newtimelineItem)
}

function generateNewStationTimeLineRow(schedulingunitTimelinerow, expandedschedulingUnitItems, station, groupingkey) {
    let newtimelineItem = { ...schedulingunitTimelinerow }
    newtimelineItem[groupingkey] = station + newtimelineItem.groupingSuffix + "#station";
    newtimelineItem.isSingleStation = true;
    newtimelineItem.id = newtimelineItem.id + newtimelineItem[groupingkey];
    cleanSingleTimelineGroups(newtimelineItem);
    expandedschedulingUnitItems.push(newtimelineItem);
}


// get rid of the other groups in this item, we do not want the  group split to be visible at the other groups.
// and we do not want the bundling anymore
function cleanSingleTimelineGroups(timelineItem) {
    delete timelineItem.group;
    delete timelineItem.project_group;
    delete timelineItem.name_group;
    delete timelineItem.priority_queue_group;
    if (timelineItem.antenna_set_group) delete timelineItem.antenna_set_group;
    if (timelineItem.band_group) delete timelineItem.band_group;
    if (timelineItem.stationsgroups) delete timelineItem.stationsgroups;
    if (timelineItem.scheduler_group) delete timelineItem.scheduler_group;
}

