import moment from "moment";
import UIConstants from "../../../utils/ui.constants";
export function getTimelineGroups() {

        return [
                { label: "No Grouping", value: "group", id: 0, title: "No Grouping" },
                { label: "Antenna Set", value: "antenna_set_group", id: 1, title: "Antenna Set" },
                { label: "Antenna Band", value: "band_group", id: 2, title: "Antenna Band" },
                { label: "Priority Queue", value: "priority_queue_group", id: 3, title: "Priority Queue" },
                { label: "Project", value: "project_group", id: 4, title: "Project" },
                { label: "Project With Tasks", value: "project_task_group", id: 5, title: "Project" },
                { label: "Station Specified", value: "station_group", id: 6, title: "Station Specified" },
                { label: "Station Used ", value: "station_used_group", id: 7, title: "Station Used" },
                { label: "Station Missing", value: "station_missing_group", id: 8, title: "Station Missing" },
                { label: "Station Group", value: "stationgroup_group", id: 9, title: "Station Group" },
                { label: "Scheduling Unit ", value: "name_group", id: 10, title: "Scheduling Unit" },
                { label: "Scheduler Type", value: "scheduler_group", id: 11, title: "Scheduler Type" }
        ]
}

export function getGroups(itemGroupKey, startTime, data, endTime) {
        let groups
        if (itemGroupKey == "group") { // then group is the default, date one
                let days = moment(endTime).diff(moment(startTime), 'days') + 1;
                groups = createDateGroups('', startTime, days);
        } else {
                groups = createGroups(data, itemGroupKey, moment(startTime), moment(endTime))
        }
        return { groups, itemGroupKey }
}

export function createGroups(timeLineRows, property, startTime, endTime) {

        const groups = [];
        if (timeLineRows !== undefined && timeLineRows.length > 0) {
                let uniquegroupnames = [...new Set(timeLineRows.map(item => item[property]))];
                uniquegroupnames = uniquegroupnames.sort((a, b) => a.localeCompare(b));

                for (const element of uniquegroupnames) {
                        if (element == undefined) continue; // no grouping data found for this item.
                        const splittedElement = element.split('#')
                        const datepart = splittedElement[1];
                        let isSingleStation = false;
                        if (splittedElement.length > 2) {
                                if (splittedElement[2] == "station") isSingleStation = true;

                        }
                        const groupDate = moment(datepart)

                        const weeknr = groupDate.format("w")
                        const weekdate = groupDate.format("MMM DD - ddd")
                        const formatedDate = groupDate.format(UIConstants.CALENDAR_GROUP_FORMAT)
                        const newgroup = { id: element, title: splittedElement[0], date: groupDate, formatedDate: formatedDate, weeknr: weeknr, weekdate: weekdate, isSingleStation: isSingleStation };
                        if (isSingleStation) {
                                newgroup.designatedStation = splittedElement[0];
                        }

                        if (groupDate >= startTime && groupDate <= endTime) groups.push(newgroup);
                }
        }

        correctLayout(groups);
        return groups;
}





// This routine is used to make the left grouping bar toggle , both by Main color (group-even/group-uneven) and by sub color (inside a group), by adding -odd
function correctLayout(groups) {
        let rowClass = ["group-even", "group-uneven"];
        let flippingrowClass = ["", "-odd"];
        let currenRowposition = 1;
        let flippingRowposition = 1;
        let previousTitle = null;
        for (let group of groups) {
                if (previousTitle !== group.title) {
                        previousTitle = group.title;
                        currenRowposition++;
                } else {
                        group.title = "";
                }
                flippingRowposition++;
                group.titleClass = rowClass[currenRowposition % 2] + flippingrowClass[flippingRowposition % 2];
        }
}

export function createDateGroups(titleprefix, startTime, amountOfGroups = 7) {
        const groups = []
        const startDate = moment(startTime).format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)
        for (let groupIndex = 0; groupIndex < amountOfGroups; groupIndex++) {
                const group = moment(startDate).add(groupIndex, "days");
                const groupTitle = group.format(UIConstants.CALENDAR_GROUP_FORMAT)
                const weeknr = group.format("w")
                const weekdate = group.format("MMM DD - ddd")
                groups.push({ id: titleprefix + groupTitle, title: titleprefix + ' ' + groupTitle, date: group, weeknr: weeknr, weekdate: weekdate })
        }

        return groups
}



export function getDefaultGrouping() {
        return getTimelineGroups()[0].value;
}

