import { render, waitFor, act, fireEvent } from "@testing-library/react";
import PanelButtons from "./PanelButtons";
import { removeReact18ConsoleErrors } from "../../../utils/test.helper";

removeReact18ConsoleErrors()

describe('PanelButtons', () => {

    const shrinkLabel = "Shrink List/Expand Timeline";
    const expandLabel = "Expand List/Shrink Timeline";
    it('toggles visibilities legend+list', () => {
        const setShowLegendbarMock = jest.fn();
        const setShowListTabsMock = jest.fn();
        let pannelComponent = null;
        act(() => {
            pannelComponent = render(<PanelButtons
                showLegendbar={false}
                setShowLegendbar={setShowLegendbarMock}
                showListTabs={true}
                setShowListTabs={setShowListTabsMock}
                showSummary={true}
                leftPanelSize={20}
                setLeftPanelSize={jest.fn()} />);
        });
        fireEvent.click(pannelComponent.getByTestId('Show Legend-button'));
        fireEvent.click(pannelComponent.getByTestId('Hide List-button'));
        expect(setShowLegendbarMock).toHaveBeenCalledWith(true);
        expect(setShowListTabsMock).toHaveBeenCalledWith(false);

    });

    it('adjusts left panel size when summary is not active', async () => {
        let setLeftPanelSizeMock = jest.fn();
        let pannelComponent = null;
        act(() => {
            pannelComponent = render(<PanelButtons
                showLegendbar={false}
                setShowLegendbar={jest.fn()}
                showListTabs={true}
                setShowListTabs={jest.fn()}
                showSummary={false}
                leftPanelSize={20}
                setLeftPanelSize={setLeftPanelSizeMock} />);
        });
        await waitFor(() => expect(pannelComponent.getByTitle(shrinkLabel).tobeinTheDocument));
        const shrinkButton = pannelComponent.getByTitle(shrinkLabel);
        fireEvent.click(shrinkButton)
        expect(setLeftPanelSizeMock).toHaveBeenCalledWith(10); // 10% shrink

        const expandButton = pannelComponent.getByTitle(expandLabel);

        fireEvent.click(expandButton)
        expect(setLeftPanelSizeMock).toHaveBeenCalledWith(30); // 10% expand
        setLeftPanelSizeMock = null;
    });


    it('disables resize buttons when summary is active', async () => {

        let pannelComponentb = null;
        act(() => {
            pannelComponentb = render(<PanelButtons
                showLegendbar={false}
                setShowLegendbar={jest.fn()}
                showListTabs={true}
                setShowListTabs={jest.fn()}
                showSummary={true}
                leftPanelSize={20}
                setLeftPanelSize={jest.fn()} />);
        });

        await waitFor(() => expect(pannelComponentb.getByTitle(shrinkLabel).tobeinTheDocument));

        const shrinkButton = pannelComponentb.getByTitle(shrinkLabel);
        expect(shrinkButton).toHaveAttribute("disabled")
        const expandButton = pannelComponentb.getByTitle(expandLabel);
        expect(expandButton).toHaveAttribute("disabled")
    });
});

