import Toggle from './Toggle'
import { MultiSelect } from "primereact/multiselect";
import { useEffect, useState } from "react";
import { updateStore } from "../../../../services/store.helper";
import UIConstants from "../../../../utils/ui.constants";
import { Panel } from 'primereact/panel';
import {
    getReservationItem,
    getTimelineItem,
    splitObjectIfSpanIsMultipleDays
} from "../../helpers/toolbar/filters.helper";
import moment from "moment";



import { fetchAllOptionsForMultiSelects } from "../../data/filters.data";
const isDebugLoggingOn = false


function filterReservations(reservations, visibleStartTime, reservationFilter, projectFilter,endTime,startTime) {
    
  
    let reservationItems = reservations.flatMap(reservation => splitObjectIfSpanIsMultipleDays(reservation, false,startTime,endTime))
        .map((reservation, index) => getReservationItem(reservation, index, moment(visibleStartTime).format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)))
        .filter(reservationItem => reservationFilter.includes(reservationItem.reason))
        

    if (projectFilter?.length) {
        reservationItems = reservationItems?.filter(reservationItem => reservationItem.project === null || projectFilter.includes(reservationItem.project))
    } else { //only show items that don't have a project
        reservationItems = reservationItems?.filter(reservationItem => reservationItem.project === null)
    }

    let datefiltered_reservationItems = reservationItems?.filter(reservationItem => reservationItem.reservation_start_time>=startTime)
    datefiltered_reservationItems = datefiltered_reservationItems?.filter(reservationItem => reservationItem.reservation_start_time<=endTime)
    
    return datefiltered_reservationItems;
}



function plainFilterSchedulingUnitItems(schedulingUnits, visibleStartTime, onSkyToggle, schedulingUnitFilter, projectFilter) {

    let schedulingUnitItems = [...schedulingUnits]
    
    if (projectFilter?.length) {
        schedulingUnitItems = schedulingUnitItems.filter(suItem => suItem.project === null || projectFilter.includes(suItem.project))
    } else {//only show items that don't have a project
        schedulingUnitItems = schedulingUnitItems.filter(suItem => suItem.project === null)
    }
    
    schedulingUnitItems = schedulingUnitItems.filter(suItem => schedulingUnitFilter.includes(suItem.status))
    return schedulingUnitItems;
}

function filterSchedulingUnits(schedulingUnits, visibleStartTime, onSkyToggle, schedulingUnitFilter, projectFilter ) {

    const  includeStationsInfo = true;
    const includeStationsgroupInfo =true;

    let schedulingUnitItems = schedulingUnits.flatMap(schedulingunit => splitObjectIfSpanIsMultipleDays(schedulingunit, onSkyToggle))
        .map(schedulingunit => getTimelineItem(schedulingunit, moment(visibleStartTime), includeStationsInfo, includeStationsgroupInfo, onSkyToggle))
        .filter(suItem => schedulingUnitFilter.includes(suItem.status))

    if (projectFilter?.length) {
        schedulingUnitItems = schedulingUnitItems.filter(suItem => suItem.project === null || projectFilter.includes(suItem.project))
    } else {//only show items that don't have a project
        schedulingUnitItems = schedulingUnitItems.filter(suItem => suItem.project === null)
    }
    return schedulingUnitItems;
}



export default function Filters(props) {
    const {
        visibleStartTime,
        endTime,
        startTime,
        setItems,
        data,
        setRefetchToggle,
        timelineStore,
        refreshGrouping,
        grouping
    } = props

    const [multiSelectAllOptions, setMultiSelectAllOptions] = useState({
        isLoading: false,
        reservationReasons: [],
        schedulingUnitStatuses: [],
        projectStatuses: [],
        projectNamesWithState: []
    })


    const [onSkyToggle, setOnSkyToggle] = useState(timelineStore.onSkyToggle ?? true)

    const [reservationsToggle, setReservationsToggle] = useState(timelineStore.reservationsToggle ?? false)
    const [reservationFilter, setReservationFilter] = useState(timelineStore.reservationFilter)

    const [schedulingUnitFilter, setSchedulingUnitFilter] = useState(timelineStore.schedulingUnitFilter)

    const [projectFilter, setProjectFilter] = useState(timelineStore.projectFilter)
    const [projectAllOptionsWithStatus, setProjectAllOptionsWithStatus] = useState([])
    const [projectStatusFilter, setProjectStatusFilter] = useState(timelineStore.projectStatusFilter ?? ["active"])


    useEffect(() => { //initializer
        fetchAllOptionsForMultiSelects(setMultiSelectAllOptions)
            .catch(e => console.error("Couldn't retrieve the multi select options for at least one multi select", e))
    }, [])

    useEffect(() => { //set default filter to all options if none in store
        if (!schedulingUnitFilter?.length) {
            setSchedulingUnitFilter(multiSelectAllOptions.schedulingUnitStatuses.map(status => status.name))
        }

        if (!reservationFilter?.length) {
            setReservationFilter(multiSelectAllOptions.reservationReasons.map(reason => reason.name))
        }

        const filteredProjectsOnState = multiSelectAllOptions?.projectNamesWithState?.filter(project => projectStatusFilter.includes(project?.project_state_value))
        setProjectAllOptionsWithStatus(filteredProjectsOnState)

        if (filteredProjectsOnState?.length > 0) {
            setProjectFilter(filteredProjectsOnState.map(project => project.name))
        }
    }, [multiSelectAllOptions])

    useEffect(() => { //the status filter is only for the project filter
        updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
            ["projectStatusFilter"]: projectStatusFilter,
        })
        const filteredProjectsOnState = multiSelectAllOptions?.projectNamesWithState?.filter(project => projectStatusFilter.includes(project?.project_state_value))
        setProjectAllOptionsWithStatus(filteredProjectsOnState)
        setProjectFilter(filteredProjectsOnState.map(project => project.name))
    }, [projectStatusFilter])

    useEffect(() => { //since reservations loading is slow and often off, only fetch them om toggle
        updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
            ["reservationsToggle"]: reservationsToggle,
        })
        timelineStore.reservationsToggle=reservationsToggle;
        if (reservationsToggle) {
            console.log(" Requesting Refresh Reservations",timelineStore ,reservationsToggle)
            setRefetchToggle(prevState => !prevState)
        }
            else {
                console.log(" Requesting to Hide Reservations",timelineStore)
                setItems(prevState => ({
                    ...prevState,
                    reservationItems: []
                }))

                fullRefresh();
            }
    
    
    }, [reservationsToggle])

    useEffect(() => {
        
        updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
            ["reservationFilter"]: reservationFilter,
            ["projectFilter"]: projectFilter,
            ["onSkyToggle"]: onSkyToggle,
        })

        let reservationsItems = []
        if (reservationsToggle) {
            if(isDebugLoggingOn) console.log("Gather Reservations for ",visibleStartTime,endTime,startTime);
            reservationsItems = filterReservations(data.reservations, visibleStartTime, reservationFilter, projectFilter,endTime,startTime);
            if (isDebugLoggingOn) console.log("Reservation sais",reservationsItems)
            setItems(prevState => ({
                ...prevState,
                reservationItems: reservationsItems
            }))
        }    else      {
            setItems(prevState => ({
                ...prevState,
                reservationItems: []
        }));
    }

      
    }, [data, reservationFilter, projectFilter])


    const fullRefresh = () => {
        const schedulingUnitItems = filterSchedulingUnits(data.schedulingUnits, visibleStartTime, onSkyToggle, schedulingUnitFilter, projectFilter);
        const plainFilteredSchedulingUnitItems = plainFilterSchedulingUnitItems(data.schedulingUnits, visibleStartTime, onSkyToggle, schedulingUnitFilter, projectFilter);
        const reservationitems =  filterReservations(data.reservations, visibleStartTime, reservationFilter, projectFilter,endTime,startTime);

        setItems(prevState => ({
            ...prevState,
            schedulingUnitItems: schedulingUnitItems,
            plainFilteredSchedulingUnitItems:plainFilteredSchedulingUnitItems

        }))
        if (reservationsToggle) {
            refreshGrouping(grouping,schedulingUnitItems,reservationitems);
        } else {
            refreshGrouping(grouping,schedulingUnitItems,[]);
        }
    }

    useEffect(() => {

        
        updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
            ["schedulingUnitFilter"]: schedulingUnitFilter,
            ["projectFilter"]: projectFilter,
            ["onSkyToggle"]: onSkyToggle,
        })

        fullRefresh();
        
    }, [data.schedulingUnits, schedulingUnitFilter, projectFilter, onSkyToggle,grouping,reservationsToggle])

  




    return <div className="p-grid timeline-view-toolbar">
        
            <Panel className="section" header="Filters">
            
                <div className="group">
                    <Toggle labelName="On Sky" tooltipText="Show on-sky system durations"
                        checkedValue={onSkyToggle} onChangeCallback={setOnSkyToggle} disabled={multiSelectAllOptions.isLoading} />
                </div>
                <div className="group">
                    <Toggle labelName={"Reservations"} tooltipText={"Show station reservations"}
                        checkedValue={reservationsToggle} onChangeCallback={setReservationsToggle}  disabled={multiSelectAllOptions.isLoading}/>
                    <div className="multiselect-container relative-position">
                        <span className="small-label">Reason:</span>
                        <MultiSelect
                            className={!reservationsToggle ? "disabled" : ""}
                            disabled={!reservationsToggle || multiSelectAllOptions.isLoading}
                            optionLabel="name"
                            optionValue="name"
                            tooltip="Select Reservation Reason(s)"
                            tooltipOptions={{ position: 'left' }}
                            value={reservationFilter}
                            options={multiSelectAllOptions.reservationReasons}
                            maxSelectedLabels="1"
                            filter
                            filterBy="name"
                            placeholder="Reason"
                            onChange={(e) => setReservationFilter(e.value)}
                        />
                    </div>
                </div>
                <div className="group">
                    <label>Scheduling Units</label>
                    <div className="multiselect-container relative-position">
                        <span className="small-label">Status:</span>
                        
                        <MultiSelect
                            optionLabel="name"
                            optionValue="name"
                            tooltip="Select Scheduling Unit Status(es)"
                            tooltipOptions={{ position: 'left' }}
                            value={schedulingUnitFilter}
                            options={multiSelectAllOptions.schedulingUnitStatuses}
                            maxSelectedLabels="1"
                            filter
                            disabled={multiSelectAllOptions.isLoading}
                            filterBy="name"
                            placeholder="Status"
                            onChange={(e) => setSchedulingUnitFilter(e.value)}
                        />
                    </div>
                </div>

                <div className="group">
                    <label>Projects</label>
                    { !multiSelectAllOptions.isLoading  &&
                    <div className="multiselect-container relative-position">
                    
                        <span className="small-label">Status:</span>
                       
                        <MultiSelect
                            optionLabel="value"
                            optionValue="value"
                            tooltip="Select Project Status(es)"
                            tooltipOptions={{ position: 'left' }}
                            value={projectStatusFilter}
                            options={multiSelectAllOptions.projectStatuses}
                            maxSelectedLabels="1"
                            filter
                            disabled={multiSelectAllOptions.isLoading}
                            filterBy="value"
                            placeholder="Status"
                            onChange={(e) => setProjectStatusFilter(e.value)}
                        />
                    </div>
}
                    <div className="multiselect-container relative-position">
                        <span className="small-label">Value:</span>
                        <MultiSelect
                            optionLabel="name"
                            optionValue="name"
                            disabled={multiSelectAllOptions.isLoading}
                            tooltip="Select Project Name(s)"
                            tooltipOptions={{ position: 'left' }}
                            value={projectFilter}
                            options={projectAllOptionsWithStatus}
                            maxSelectedLabels="1"
                            filter
                            filterBy="name"
                            placeholder="Project"
                            onChange={(e) => setProjectFilter(e.value)}
                        />
                    </div>
                </div>
            
            
            </Panel>
        
    </div>
}