import { useEffect, useState } from 'react';
import { Button } from 'primereact/button';
import { Dialog } from 'primereact/dialog';
import ViewTable from './../../components/ViewTable';
import { Link } from 'react-router-dom';
import WorkflowService from '../../services/workflow.service';
import TaskAssigner from './task.assigner';
import IngestProgress from '../Scheduling/ingest.progress';


const UnPinData = ({ tasks, schedulingUnit, onCancel, ...props }) => {
    const [showConfirmDialog, setShowConfirmDialog] = useState(false);
    const [currentWorkflowTask, setCurrentWorkflowTask] = useState();
    const [processPermission, setProcessPermission] = useState()
    const [assignTo, setAssignTo] = useState();
    const [reassign, setReassign] = useState(false);
    const defaultcolumns = [{
        name: "Name",
        totalDataSize: "Total Data Size(TB)",
        dataSizeNotDeleted: "Data Size on Disk(TB)"
    }];
    const optionalcolumns = [{
        actionpath: "actionpath",
    }];
    const defaultSortColumn = [{ name: "Name", desc: true }];
    const columnclassname = [{
        "Name": "filter-input-150", "Total Data Size(TB)": "filter-input-100", "Data Size Not Deleted(TB)": "filter-input-100"
    }];
    const toggleDialog = () => {
        setShowConfirmDialog(!showConfirmDialog)
    };
    const user = JSON.parse(localStorage.getItem("user"));
    /**
    * Method will trigger on click next buton
    * here onNext props coming from parent, where will handle redirection to other page
    */
    const Next = async () => {
        await props.getCurrentTaskDetails();
        const promise = [];
        promise.push(WorkflowService.updateQA_Perform(props.id, {}));
        Promise.all(promise).then((responses) => {
            if (responses.indexOf(null) < 0) {
                props.onNext();
            } else {
                props.onError('error', 'Unable to proceed', 'Please clear your browser cookies and try again');
            }
        });
        setShowConfirmDialog(false)
    }

    useEffect(() => {
        if (!props.readOnly && !currentWorkflowTask) {
            props.getCurrentTaskDetails()
                .then(currentWorkflowTask => {
                    if (currentWorkflowTask!==undefined) {
                        setCurrentWorkflowTask(currentWorkflowTask);
                    }
                    
                });
            props.processPermission()
                .then(permission => {
                    setProcessPermission(permission)
                })
        }
        if (props.readOnly && !assignTo &&props.workflowTask!==undefined) {
            setAssignTo(props.workflowTask.owner);
        }
    }, []);

    /**
     * Callback function passed to the Task Assigner and is called after assigning or reassigning task owner.
     */
    const assignTaskCB = async () => {
        let currentWorkflowTask = await props.getCurrentTaskDetails();
        setCurrentWorkflowTask(currentWorkflowTask);
        setAssignTo(currentWorkflowTask.owner_id);
        setReassign(false);
    }
    return (
        <div className="p-fluid mt-2">
            <div className="p-field p-grid workflow-wrap" style={{ paddingLeft: '-10px' }}>
                <label htmlFor="assignTo" className="col-lg-2 col-md-2 col-sm-12">Assign To</label>
                <div className="col-lg-3 col-md-3 col-sm-12" data-testid="assignTo" >
                    {/* Display the assigned owner of the task */}
                    {assignTo &&
                        <>{!props.readOnly ? currentWorkflowTask.owner_username : props.workflowTask.owner_username}
                            {/* Allow to edit the owner if the task is not yet completed */}
                            {!props.readOnly && !reassign &&
                                <>
                                    {currentWorkflowTask?.editPermissions
                                        ? <Link onClick={ () => setReassign(true)} style={{ marginLeft: '10px' }}>
                                            <i className="pi pi-pencil"></i>
                                        </Link>
                                        : <Link style={{ cursor: 'default', pointerEvents: "none", color: 'grey', marginLeft: '10px' }} onClick={ ()=>setReassign(true)}>
                                            <i className="pi pi-pencil"></i>
                                        </Link>

                                    }
                                </>}
                        </>
                    }
                    {/* Display the task assigner if owner is not assigned or to reassign to other user */}
                    {(!assignTo || reassign) && currentWorkflowTask &&
                        <TaskAssigner currentWorkflowTask={currentWorkflowTask} disableAssignToMe={currentWorkflowTask && currentWorkflowTask.owner_username === user.name ? true : false}
                            projectRoles={props.projectRoles} callback={assignTaskCB} />
                    }
                </div>
                {props.showIngestProgress &&
                    <><label className="col-lg-2 col-md-2 col-sm-12" >Ingest Completeness</label>
                        <span className="col-lg-4 col-md-4 col-sm-12">
                            <IngestProgress schedulingUnitId={schedulingUnit.id} />
                        </span></>}
            </div>
            <label><h6>Details of data products of Tasks</h6></label>
            <div style={{ marginTop: "-50px" }}>
                <ViewTable
                    data={tasks.filter(task => (task.totalDataSize !== undefined || task.dataSizeNotDeleted !== undefined))}
                    optionalcolumns={optionalcolumns}
                    defaultcolumns={defaultcolumns}
                    defaultSortColumn={defaultSortColumn}
                    columnclassname={columnclassname}
                    showColumnFilter={false}
                    showGlobalFilter={false}
                    showTopTotal={false}
                    allowColumnSelection={false}
                    showaction="true"
                    keyaccessor="id"
                    defaultpagesize={tasks.length}
                />
            </div>
            <div className="p-grid p-justify-start mt-2">
                {!props.readOnly && <div className="btn-bar">
                    <Button
                        disabled={processPermission && currentWorkflowTask && currentWorkflowTask.editPermissions && currentWorkflowTask.owner_username === user.name ? false : true}
                        label="Delete" className="p-button-primary" icon="pi pi-trash" onClick={toggleDialog} />
                </div>}
                {!props.readOnly && <div className="btn-bar">
                    <Button label="Cancel" className="p-button-danger" icon="pi pi-times" style={{ width: '90px' }}
                        onClick={ onCancel() } />
                </div>}
            </div>
            {!props.readOnly &&
                <>
                    {currentWorkflowTask && currentWorkflowTask.editPermissions && currentWorkflowTask.owner_username === user.name ?
                        <span></span> : <span style={{ color: 'red' }}>* You can only delete if you are assigned to this workflow step</span>}
                </>
            }
            <div className="p-grid" data-testid="confirm_dialog">
                <Dialog header={'Confirm'} visible={showConfirmDialog} style={{ width: '40vw' }} inputId="confirm_dialog"
                    modal={true} onHide={() => setShowConfirmDialog(false)}
                    footer={<div>
                        <Button key="back" onClick={Next} label="Yes" />
                        <Button key="submit" type="primary" onClick={() => setShowConfirmDialog(false)} label="No" />
                    </div>
                    } >
                    <div className="p-grid">
                        <div className="col-lg-2 col-md-2 col-sm-2" style={{ margin: 'auto' }}>
                            <i className="pi pi-large pi-question-circle pi-warning"></i>
                        </div>
                        <div className="col-lg-10 col-md-10 col-sm-10">
                            Are you sure you want to delete dataproducts for scheduling unit {schedulingUnit.id} - {schedulingUnit.name} - {schedulingUnit.description} with status {schedulingUnit.status}
                        </div>
                    </div>
                </Dialog>
            </div>
        </div>
    )
}
export default UnPinData;