import { render, act, fireEvent, waitFor  } from '@testing-library/react';
import App from '../App';
import { MakeWeekViewPrimaryMock, MockAxiosSecurity } from './axiosmocks.';
import MockAdapter from 'axios-mock-adapter';
import axios from 'axios';
import PermissionStackUtil from '../authenticate/permission.stack.handler';
import subextended from '../__mocks__/scheduling_unit_blueprint_extended.json'
import schedulingunitblueprint from '../__mocks__/scheduling_unit_blueprint'
import { giveConsistentNodes } from '../utils/test.helper';


function MakePrimaryMock() {
  let mock = new MockAdapter(axios);
  mock.onGet("/api/authentication_state/").reply(200, { is_authenticated: true, username: "a username", csrftoken: "a csrftoken", websocket_token: "a websocket tokcen" }, {
    'Set-Cookie': "srftoken=abcdefgh; expires=Sun, 22 Dec 2324 22:58:11 GMT;  Path=/; SameSite=Lax"
  });
  const urlPattern = /^\/api\/util\/lst.*$/;
  mock.onGet(urlPattern).reply(200, { "UTC": "2023-12-24T00:00:00Z", "LST": { "CS002": "06:36:32" } });
  mock.onGet("/api/util/utc").reply(200, "2023-12-24T22:58:22.413683");
 
  mock.onOptions("/api/scheduling_unit_blueprint_extended/").reply(200, subextended);
  mock.onGet("/api/scheduling_unit_blueprint/3997",schedulingunitblueprint) // search result
  MakeWeekViewPrimaryMock(mock); // default page opens at weekview
  return mock;
}
describe('App Search', () => {
  it('Searches Correctly for Scheduling Unit', async () => {
    // Render the App component
    MockAxiosSecurity(MakePrimaryMock());
    await PermissionStackUtil.getPermissions(true);
    let component
    await act(async () => { component = render(<App />); });


    await waitFor(() => {
      expect(component.queryAllByTitle('Enter Object Id to search Object')).toHaveLength(1)});

      const inputElement = component.queryAllByTitle('Enter Object Id to search Object')[0];
      fireEvent.change(inputElement, {target: {value: '3997'}})
      //act( async() => { await userEvent.type(inputElement, '484{enter}');});

      
      
    //  const inputdropdownPlaceholder = component.queryAllByTitle('Select type to search for')[0];
    //  fireEvent.change(inputdropdownPlaceholder, {target: {value: 'Task'}})
      
      const searchButton = component.queryAllByTitle('Start search')[0];
      act(() => { fireEvent.click(searchButton); });
      const componentSnapshot = giveConsistentNodes(component.container.outerHTML)
      expect(componentSnapshot).toMatchSnapshot("After  Search");
    });
  });

  describe('App', () => {


    it('renders the App component without crashing', () => {
      // Render the App component
      render(<App />);

      // You can add more assertions here if needed
      expect(document.querySelector('.App')).toBeInTheDocument();
    });


    it('renders the App component with Redirection to  login', async () => {
      // Render the App component
      MockAxiosSecurity(MakePrimaryMock());
      render(<App />);

      let component
      await act(async () => { component = render(<App />); });
      expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot();
      component.unmount();
    });





    it('renders the App component with Valid Credentials', async () => {
      // Render the App component
      MockAxiosSecurity(MakePrimaryMock());
      await PermissionStackUtil.getPermissions(true);
      render(<App />);

      let component
      await act(async () => { component = render(<App />); });
      expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot("Valid Credentials");



      await waitFor(() => { expect(component.queryAllByTitle('Logout a username')).toHaveLength(2); }, { timeout: 5000 });


      let navigationMenuItem = component.container.querySelector(".p-button.p-component.nav-btn.nav-btn-tooltip");
      fireEvent.click(navigationMenuItem);
      expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot("After Menu item click");




      let ToggleMenuoButton = component.queryAllByTitle("Toggle Menu")[0];
      fireEvent.click(ToggleMenuoButton);
      expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot("After Menu Toggle One");
      fireEvent.click(ToggleMenuoButton);
      expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot("After Menu Toggle Two");
      

      let logoutButton = component.queryAllByTitle("Logout a username")[0];
      fireEvent.click(logoutButton);
      await waitFor(() => { expect(component.queryAllByTitle('Logout a username')).toHaveLength(1); }, { timeout: 5000 });
      
      expect(giveConsistentNodes(component.container.outerHTML)).toMatchSnapshot("After Logout");

      component.unmount();
    });
  });