

import axios from "axios";
import MockAdapter from 'axios-mock-adapter';
import { render, act,waitFor } from '@testing-library/react';
import reservationview from '../__mocks__/reservation.view.json'
import reservationtemplateeight from '../__mocks__/reservation.template.eight.json'
import reservationtemplatenine from '../__mocks__/reservation.template.nine.json'


import { MockAxiosSecurity  } from './axiosmocks.'
import { MemoryRouter } from "react-router-dom";
import PermissionStackUtil from "../authenticate/permission.stack.handler";
import { ReservationView } from "../routes/Reservation/reservation.view";




function MakePrimaryMock() {
    let mock = new MockAdapter(axios);
     mock.onGet("/api/reservation/14311").reply(200, reservationview);
     mock.onGet("/api/reservation_template/8").reply(200, reservationtemplateeight);
     mock.onGet("/api/reservation_template/9").reply(200, reservationtemplatenine);
     
    return mock

}

describe('Reservation View ', () => {

    it('Reservation View  renders correctly without proper security', async () => {
        MakePrimaryMock();
        let component
        await act(async () => { component = render(<MemoryRouter><ReservationView  match={{params:{id:14311}}}  history={{}} location="/reservation/view/14311" /></MemoryRouter>) });
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        component.unmount();

    });

    it('Reservation View  correctly with data', async () => {
        let {  component } = await RenderReservationView();
        const componentSnapshot = component.asFragment();
        expect(componentSnapshot).toMatchSnapshot();
        await waitFor(() => { expect(component.getByText('Reservation Parameters')).toBeInTheDocument(); }, { timeout: 5000 });
        component.unmount();

    });
  
});



async function RenderReservationView() {
    MockAxiosSecurity(MakePrimaryMock());
    await PermissionStackUtil.getPermissions(true);
    let component;
    let taskListInstance;
    await act(async () => {
        component = render(<MemoryRouter><ReservationView match={{params:{id:14311}}}  history={{}} location="/reservation/view/14311" ref={(instance) => { taskListInstance = instance; } } /></MemoryRouter>);
    });
    return { taskListInstance, component };
}

