import { cleanup, fireEvent,  act, waitFor } from '@testing-library/react';
import renderer from 'react-test-renderer';

import '@testing-library/jest-dom';
import { SchedulingUnitCreate } from './create';
import ScheduleService from '../../services/schedule.service';
import UtilService from '../../services/util.service';
import ProjectService from '../../services/project.service';

import TaskService from '../../services/task.service';
import SUServiceMock from '../../__mocks__/scheduleunit.service.data';
import ProjectServiceMock from '../../__mocks__/project.service.data';
import TaskServiceMock from '../../__mocks__/task.service.data';
import UtilServiceMock from '../../__mocks__/util.service.data';
import { removeReact18ConsoleErrors, renderPage, setMultiSelectValue } from "../../utils/test.helper";
// import { setSchedulingUnitStrategy } from "./schedulingunit.test.helper";

let projectListSpy, observStrategiesSpy, taskTemplatesSpy, utilSpy, taskFilterDefSpy, suConstraintTemplateSpy,
    stationGroupSpy, templatePurposesSpy, templateStatesSpy, rolesSpy //, scheduleSetByIdSpy;

//Jest issue: https://github.com/jestjs/jest/issues/9709: Cannot be called in beforeXX test method
// const OBSERVATION_STRATEGY_TEMPLATES = SUServiceMock.getObservStrategies()

jest.setTimeout(120000);
removeReact18ConsoleErrors();


function createStandardPageMocks() {
    projectListSpy = jest.spyOn(ProjectService, 'getProjectList').mockImplementation(() => {
        return Promise.resolve(ProjectServiceMock.projectList)
    });
    observStrategiesSpy = jest.spyOn(ScheduleService, 'getObservationStrategies').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.getObservStrategies())
    });
    taskTemplatesSpy = jest.spyOn(TaskService, 'getTaskTemplates').mockImplementation(() => {
        return Promise.resolve(TaskServiceMock.getTaskTemplates())
    });
    suConstraintTemplateSpy = jest.spyOn(ScheduleService, 'getSchedulingConstraintTemplates').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.getSUCTemplates());
    });
    rolesSpy = jest.spyOn(ProjectService, 'getMyRoles').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.getMyRoles)
    });
    stationGroupSpy = jest.spyOn(ScheduleService, 'getStationGroup').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.getStationGroup)
    });
    utilSpy = jest.spyOn(UtilService, 'getPriorityQueueType').mockImplementation(() => {
        return Promise.resolve(UtilServiceMock.getPriorityQueueType)
    })
    taskFilterDefSpy = jest.spyOn(TaskService, 'getTaskFilterDefinition').mockImplementation(() => {
        return Promise.resolve(TaskServiceMock.getTaskFilterDefinition)
    });
    templatePurposesSpy = jest.spyOn(ScheduleService, 'getStrategyPurpose').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.templatePurposes)
    });
    templateStatesSpy = jest.spyOn(ScheduleService, 'getStrategyState').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.templateStates)
    });
    /*
    scheduleSetByIdSpy = jest.spyOn(ScheduleService, 'getSchedulingBySet').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.getSchedulingBySet())
    });
    */
}

async function setSchedulingUnitBasics(pageContent) {
    const nameElement = pageContent.queryByTestId('name');
    act(() => fireEvent.change(nameElement, { target: { value: "Wolverine" } }));
    const descriptionElement = pageContent.queryByTestId('description');
    act(() => fireEvent.change(descriptionElement, { target: { value: "Big claws" } }));
    await setMultiSelectValue(pageContent, "project", "high")
    await setMultiSelectValue(pageContent, "schedSet", "Test Scheduling Set")
}

describe('Scheduling Unit create page default', () => {


    beforeEach(async () => {
        createStandardPageMocks();

    });

    afterEach(() => {
        jest.clearAllMocks();
        cleanup();
    });

    test("Renders with all standard service mocks", async () => {

        const page = renderer
            .create(<SchedulingUnitCreate location={{ pathname: '/schedulingunit/create' }} />)
            .toJSON();


            await waitFor (() => expect(page).toMatchSnapshot()            );
    })




test("Verify Mock Calls", async () => {
    await renderPage(<SchedulingUnitCreate location={{ pathname: '/schedulingunit/create' }} />);

    expect(projectListSpy).toHaveBeenCalled();
    expect(observStrategiesSpy).toHaveBeenCalled();
    expect(suConstraintTemplateSpy).toHaveBeenCalled();
    expect(rolesSpy).toHaveBeenCalled();
    expect(taskTemplatesSpy).toHaveBeenCalled();
    expect(stationGroupSpy).toHaveBeenCalled();
    expect(utilSpy).toHaveBeenCalled();
    expect(taskFilterDefSpy).toHaveBeenCalled();
    expect(templatePurposesSpy).toHaveBeenCalled();
    expect(templateStatesSpy).toHaveBeenCalled();
});


test("Changes a text field", async () => {
    let pageContent = await renderPage(<SchedulingUnitCreate location={{ pathname: '/schedulingunit/create' }} />);
    await waitFor (() => expect (pageContent.queryByTestId('name')).toBeInTheDocument());
    const nameInputEl = pageContent.queryByTestId('name');

    fireEvent.change(nameInputEl, {target: {value: 'SU High 1'}});
     expect(nameInputEl.value).toBe("SU High 1");

});



test("Selects a project, loads the project's sets and selects the test scheduling set", async () => {
    let pageContent = await renderPage(<SchedulingUnitCreate location={{ pathname: '/schedulingunit/create' }} />);
    let scheduleSetListSpy = jest.spyOn(ScheduleService, 'getSchedulingSets').mockImplementation((filter) => {
        let allSUSets = SUServiceMock.scheduleSetList;
        let project = filter.replace("project=", "");
        return Promise.resolve(allSUSets.filter(set => set.project_id === project));
    });
    await waitFor (() => expect (pageContent.queryByTestId('name')).toBeInTheDocument());
    await setSchedulingUnitBasics(pageContent);
    expect(scheduleSetListSpy).toHaveBeenCalled()
    expect(pageContent.queryByTestId("project").textContent).toContain("high");
    expect(pageContent.queryByTestId("schedSet").textContent).toContain("Test Scheduling Set");
    expect(pageContent.queryByTestId('save-btn')).toHaveAttribute("disabled");
});

});
/*
// TODO : Fix Unit Test Scheduling Unit create page with an observation strategy template (multiple)
describe('Scheduling Unit create page with an observation strategy template', () => {
    let pageContent, saveButton;
    let scheduleSetListSpy, stationSpy, utcSpy, validatorSpy, stationGroupSpy;

    function createTemplatePageMocks() {
        scheduleSetListSpy = jest.spyOn(ScheduleService, 'getSchedulingSets').mockImplementation((filter) => {
            let allSUSets = SUServiceMock.scheduleSetList;
            let project = filter.replace("project=", "");
            return Promise.resolve(allSUSets.filter(set => set.project_id === project));
        });
        stationSpy = jest.spyOn(ScheduleService, 'getStations').mockImplementation(() => {
            return Promise.resolve(SUServiceMock.getStations)
        });
        utcSpy = jest.spyOn(UtilService, 'getUTC').mockImplementation(() => {
            return Promise.resolve(UtilServiceMock.getUTC)
        });
        validatorSpy = jest.spyOn(ScheduleService, 'validateSpecificationsDoc').mockImplementation(() => {
            return Promise.resolve({valid: true, message: "validation_result['message']"})
        });
        stationGroupSpy = jest.spyOn(ScheduleService, 'getStationGroup').mockImplementation(() => {
            return Promise.resolve(SUServiceMock.getStationGroup)
        });
    }

    beforeEach(async () => {
        createStandardPageMocks();
        createTemplatePageMocks();
        pageContent = await renderPage(<SchedulingUnitCreate location={{pathname: '/schedulingunit/create'}}/>);
        saveButton = pageContent.queryByTestId('save-btn');
    })

    afterEach(() => {
        jest.clearAllMocks();
        cleanup()
    });

    async function  waitForAndCheckTemplateMocks() {

        // because react can be executing in the background, let's give the system actual some momentum to call the functions with waitfor
       
        
        await waitFor(() =>expect(scheduleSetListSpy).toHaveBeenCalled());
        
        await waitFor(() => expect(stationSpy).toHaveBeenCalled());
        await waitFor(() =>expect(utcSpy).toHaveBeenCalled());
        await waitFor(() =>expect(validatorSpy).toHaveBeenCalled());
        await waitFor(() =>expect(scheduleSetByIdSpy).toHaveBeenCalled());
        await waitFor(() =>expect(stationGroupSpy).toHaveBeenCalled());
    }

    function checkCorrectSpecifications() {
        expect(pageContent.queryByTestId('Stations_stations'), "No stations defined").toBeDefined()
        try {
            expect(screen.getByText("Constraints specification is valid")).toBeDefined();
            expect(screen.getByText("Task Parameters specification is valid")).toBeDefined();
        } catch (err) {
            const validationMessageElement = screen.getByTestId('validation-spec-msg');
            expect(validationMessageElement.className, "It is not an error message so earlier asserts should not have failed").toBe('error')
            expect(validationMessageElement.textContent, "This expect should fail to give an indication of the error in the template").toBe('ignore this')
        }
    }
//var singleobs  =  [OBSERVATION_STRATEGY_TEMPLATES[0]];
    test.each(OBSERVATION_STRATEGY_TEMPLATES)(`Correctly creates, saves and checks constraints for template '$name'`, async (observationStrategy) => {
        //TODO: increase test coverage to include previous tests https://support.astron.nl/jira/browse/TMSS-2537
        
        await setSchedulingUnitBasics(pageContent);
        await setSchedulingUnitStrategy(observationStrategy, pageContent);
        
        await waitForAndCheckTemplateMocks();
        checkCorrectSpecifications();

        expect(saveButton.hasAttribute("disabled"),
            "Saving is disabled, but shouldn't for template: " + observationStrategy.name).toBeFalsy();
        let saveSUFromStrategySpy = jest.spyOn(ScheduleService, 'saveSchedulingUnit').mockImplementation(() => {
            let schedulingUnitDraft = SUServiceMock.getSchedulingUnitFromObservStrategy(observationStrategy);
            schedulingUnitDraft["isSUUpdated"] = true;
            return Promise.resolve(schedulingUnitDraft);
        });

        await clickItem(saveButton)
        expect(saveSUFromStrategySpy).toHaveBeenCalled();
    });
});
*/